# coding=utf-8

"""
Collects all number values from the db.serverStatus() command, other
values are ignored.

**Note:** this collector expects pymongo 2.4 and onward. See the pymongo
changelog for more details:
http://api.mongodb.org/python/current/changelog.html#changes-in-version-2-4

#### Dependencies

 * pymongo

#### Example Configuration

MongoDBCollector.conf

```
    enabled = True
    hosts = localhost:27017, alias1@localhost:27018, etc
```
"""

import diamond.collector
from diamond.collector import str_to_bool
import re
import zlib

try:
    import pymongo
except ImportError:
    pymongo = None

try:
    from pymongo import ReadPreference
except ImportError:
    ReadPreference = None


class MongoDBCollector(diamond.collector.Collector):
    MAX_CRC32 = 4294967295

    def __init__(self, *args, **kwargs):
        self.__totals = {}
        super(MongoDBCollector, self).__init__(*args, **kwargs)

    def get_default_config_help(self):
        config_help = super(MongoDBCollector, self).get_default_config_help()
        config_help.update({
            'hosts': 'Array of hostname(:port) elements to get metrics from'
                     'Set an alias by prefixing host:port with alias@',
            'host': 'A single hostname(:port) to get metrics from'
                    ' (can be used instead of hosts and overrides it)',
            'user': 'Username for authenticated login (optional)',
            'passwd': 'Password for authenticated login (optional)',
            'databases': 'A regex of which databases to gather metrics for.'
                         ' Defaults to all databases.',
            'ignore_collections': 'A regex of which collections to ignore.'
                                  ' MapReduce temporary collections (tmp.mr.*)'
                                  ' are ignored by default.',
            'collection_sample_rate': 'Only send stats for a consistent subset '
            'of collections. This is applied after collections are ignored via'
            ' ignore_collections Sampling uses crc32 so it is consistent across'
            ' replicas. Value between 0 and 1. Default is 1',
            'network_timeout': 'Timeout for mongodb connection (in'
                               ' milliseconds). There is no timeout by'
                               ' default.',
            'simple': 'Only collect the same metrics as mongostat.',
            'translate_collections': 'Translate dot (.) to underscores (_)'
                                     ' in collection names.',
            'ssl': 'True to enable SSL connections to the MongoDB server.'
                    ' Default is False',
            'replica': 'True to enable replica set logging. Reports health of'
                       ' individual nodes as well as basic aggregate stats.'
                       ' Default is False'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(MongoDBCollector, self).get_default_config()
        config.update({
            'path':      'mongo',
            'hosts':     ['localhost'],
            'user':      None,
            'passwd':      None,
            'databases': '.*',
            'ignore_collections': '^tmp\.mr\.',
            'network_timeout': None,
            'simple': 'False',
            'translate_collections': 'False',
            'collection_sample_rate': 1,
            'ssl': False,
            'replica': False
        })
        return config

    def collect(self):
        """Collect number values from db.serverStatus()"""

        if pymongo is None:
            self.log.error('Unable to import pymongo')
            return

        hosts = self.config.get('hosts')

        # Convert a string config value to be an array
        if isinstance(hosts, basestring):
            hosts = [hosts]

        # we need this for backwards compatibility
        if 'host' in self.config:
            hosts = [self.config['host']]

        # convert network_timeout to integer
        if self.config['network_timeout']:
            self.config['network_timeout'] = int(
                self.config['network_timeout'])

        # convert collection_sample_rate to float
        if self.config['collection_sample_rate']:
            self.config['collection_sample_rate'] = float(
                self.config['collection_sample_rate'])

        # use auth if given
        if 'user' in self.config:
            user = self.config['user']
        else:
            user = None

        if 'passwd' in self.config:
            passwd = self.config['passwd']
        else:
            passwd = None

        for host in hosts:
            matches = re.search('((.+)\@)?(.+)?', host)
            alias = matches.group(2)
            host = matches.group(3)

            if alias is None:
                if len(hosts) == 1:
                    # one host only, no need to have a prefix
                    base_prefix = []
                else:
                    base_prefix = [re.sub('[:\.]', '_', host)]
            else:
                base_prefix = [alias]

            try:
                # Ensure that the SSL option is a boolean.
                if type(self.config['ssl']) is str:
                    self.config['ssl'] = str_to_bool(self.config['ssl'])

                if ReadPreference is None:
                    conn = pymongo.MongoClient(
                        host,
                        socketTimeoutMS=self.config['network_timeout'],
                        ssl=self.config['ssl'],
                    )
                else:
                    conn = pymongo.MongoClient(
                        host,
                        socketTimeoutMS=self.config['network_timeout'],
                        ssl=self.config['ssl'],
                        read_preference=ReadPreference.SECONDARY,
                    )
            except Exception, e:
                self.log.error('Couldnt connect to mongodb: %s', e)
                continue

            # try auth
            if user:
                try:
                    conn.admin.authenticate(user, passwd)
                except Exception, e:
                    self.log.error(
                        'User auth given, but could not autheticate' +
                        ' with host: %s, err: %s' % (host, e))
                    return{}

            data = conn.db.command('serverStatus')
            self._publish_transformed(data, base_prefix)
            if str_to_bool(self.config['simple']):
                data = self._extract_simple_data(data)
            if str_to_bool(self.config['replica']):
                try:
                    replset_data = conn.admin.command('replSetGetStatus')
                    self._publish_replset(replset_data, base_prefix)
                except pymongo.errors.OperationFailure as e:
                    self.log.error('error getting replica set status', e)
            self._publish_transformed(data, base_prefix)

            self._publish_dict_with_prefix(data, base_prefix)
            db_name_filter = re.compile(self.config['databases'])
            ignored_collections = re.compile(self.config['ignore_collections'])
            sample_threshold = self.MAX_CRC32 * self.config[
                'collection_sample_rate']
            for db_name in conn.database_names():
                if not db_name_filter.search(db_name):
                    continue
                db_stats = conn[db_name].command('dbStats')
                db_prefix = base_prefix + ['databases', db_name]
                self._publish_dict_with_prefix(db_stats, db_prefix)
                for collection_name in conn[db_name].collection_names():
                    if ignored_collections.search(collection_name):
                        continue
                    if (self.config['collection_sample_rate'] < 1 and (
                            zlib.crc32(collection_name) & 0xffffffff
                    ) > sample_threshold):
                        continue

                    collection_stats = conn[db_name].command('collstats',
                                                             collection_name)
                    if str_to_bool(self.config['translate_collections']):
                        collection_name = collection_name.replace('.', '_')
                    collection_prefix = db_prefix + [collection_name]
                    self._publish_dict_with_prefix(collection_stats,
                                                   collection_prefix)

    def _publish_replset(self, data, base_prefix):
        """ Given a response to replSetGetStatus, publishes all numeric values
            of the instance, aggregate stats of healthy nodes vs total nodes,
            and the observed statuses of all nodes in the replica set.
        """
        prefix = base_prefix + ['replset']
        self._publish_dict_with_prefix(data, prefix)
        total_nodes = len(data['members'])
        healthy_nodes = reduce(lambda value, node: value + node['health'],
                               data['members'], 0)

        self._publish_dict_with_prefix({
            'healthy_nodes': healthy_nodes,
            'total_nodes': total_nodes
        }, prefix)
        for node in data['members']:
            self._publish_dict_with_prefix(node,
                                           prefix + ['node', str(node['_id'])])

    def _publish_transformed(self, data, base_prefix):
        """ Publish values of type: counter or percent """
        self._publish_dict_with_prefix(data.get('opcounters', {}),
                                       base_prefix + ['opcounters_per_sec'],
                                       self.publish_counter)
        self._publish_dict_with_prefix(data.get('opcountersRepl', {}),
                                       base_prefix +
                                       ['opcountersRepl_per_sec'],
                                       self.publish_counter)
        self._publish_metrics(base_prefix + ['backgroundFlushing_per_sec'],
                              'flushes',
                              data.get('backgroundFlushing', {}),
                              self.publish_counter)
        self._publish_dict_with_prefix(data.get('network', {}),
                                       base_prefix + ['network_per_sec'],
                                       self.publish_counter)
        self._publish_metrics(base_prefix + ['extra_info_per_sec'],
                              'page_faults',
                              data.get('extra_info', {}),
                              self.publish_counter)

        def get_dotted_value(data, key_name):
            key_name = key_name.split('.')
            for i in key_name:
                data = data.get(i, {})
                if not data:
                    return 0
            return data

        def compute_interval(data, total_name):
            current_total = get_dotted_value(data, total_name)
            total_key = '.'.join(base_prefix + [total_name])
            last_total = self.__totals.get(total_key, current_total)
            interval = current_total - last_total
            self.__totals[total_key] = current_total
            return interval

        def publish_percent(value_name, total_name, data):
            value = float(get_dotted_value(data, value_name) * 100)
            interval = compute_interval(data, total_name)
            key = '.'.join(base_prefix + ['percent', value_name])
            self.publish_counter(key, value, time_delta=bool(interval),
                                 interval=interval)

        publish_percent('globalLock.lockTime', 'globalLock.totalTime', data)
        publish_percent('indexCounters.btree.misses',
                        'indexCounters.btree.accesses', data)

        locks = data.get('locks')
        if locks:
            if '.' in locks:
                locks['_global_'] = locks['.']
                del (locks['.'])
            key_prefix = '.'.join(base_prefix + ['percent'])
            db_name_filter = re.compile(self.config['databases'])
            interval = compute_interval(data, 'uptimeMillis')
            for db_name in locks:
                if not db_name_filter.search(db_name):
                    continue
                r = get_dotted_value(
                    locks,
                    '%s.timeLockedMicros.r' % db_name)
                R = get_dotted_value(
                    locks,
                    '.%s.timeLockedMicros.R' % db_name)
                value = float(r + R) / 10
                if value:
                    self.publish_counter(
                        key_prefix + '.locks.%s.read' % db_name,
                        value, time_delta=bool(interval),
                        interval=interval)
                w = get_dotted_value(
                    locks,
                    '%s.timeLockedMicros.w' % db_name)
                W = get_dotted_value(
                    locks,
                    '%s.timeLockedMicros.W' % db_name)
                value = float(w + W) / 10
                if value:
                    self.publish_counter(
                        key_prefix + '.locks.%s.write' % db_name,
                        value, time_delta=bool(interval), interval=interval)

    def _publish_dict_with_prefix(self, dict, prefix, publishfn=None):
        for key in dict:
            self._publish_metrics(prefix, key, dict, publishfn)

    def _publish_metrics(self, prev_keys, key, data, publishfn=None):
        """Recursively publish keys"""
        if key not in data:
            return
        value = data[key]
        keys = prev_keys + [key]
        keys = [x.replace(" ", "_").replace("-", ".") for x in keys]
        if not publishfn:
            publishfn = self.publish
        if isinstance(value, dict):
            for new_key in value:
                self._publish_metrics(keys, new_key, value)
        elif isinstance(value, int) or isinstance(value, float):
            publishfn('.'.join(keys), value)
        elif isinstance(value, long):
            publishfn('.'.join(keys), float(value))

    def _extract_simple_data(self, data):
        return {
            'connections': data.get('connections'),
            'globalLock': data.get('globalLock'),
            'indexCounters': data.get('indexCounters')
        }



# coding=utf-8

"""
The SoftInterruptCollector collects metrics on software interrupts from
/proc/stat

#### Dependencies

 * /proc/stat

"""

import platform
import os
import diamond.collector

# Detect the architecture of the system
# and set the counters for MAX_VALUES
# appropriately. Otherwise, rolling over
# counters will cause incorrect or
# negative values.
if platform.architecture()[0] == '64bit':
    counter = (2 ** 64) - 1
else:
    counter = (2 ** 32) - 1


class SoftInterruptCollector(diamond.collector.Collector):

    PROC = '/proc/stat'

    def get_default_config_help(self):
        config_help = super(SoftInterruptCollector,
                            self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(SoftInterruptCollector, self).get_default_config()
        config.update({
            'path':     'softirq'
        })
        return config

    def collect(self):
        """
        Collect interrupt data
        """
        if not os.access(self.PROC, os.R_OK):
            return False

        # Open PROC file
        file = open(self.PROC, 'r')

        # Get data
        for line in file:

            if not line.startswith('softirq'):
                continue

            data = line.split()

            metric_name = 'total'
            metric_value = int(data[1])
            metric_value = int(self.derivative(
                metric_name,
                long(metric_value), counter))
            self.publish(metric_name, metric_value)

            for i in range(2, len(data)):
                metric_name = str(i - 2)
                metric_value = int(data[i])
                metric_value = int(self.derivative(
                    metric_name,
                    long(metric_value), counter))
                self.publish(metric_name, metric_value)

        # Close file
        file.close()



# coding=utf-8

"""
Collects data from one or more Redis Servers

#### Dependencies

 * redis

#### Notes

The collector is named an odd redisstat because of an import issue with
having the python library called redis and this collector's module being called
redis, so we use an odd name for this collector. This doesn't affect the usage
of this collector.

Example config file RedisCollector.conf

```
enabled=True
host=redis.example.com
port=16379
auth=PASSWORD
```

or for multi-instance mode:

```
enabled=True
instances = nick1@host1:port1, nick2@host2:port2/PASSWORD, ...
```

For connecting via unix sockets, provide the path prefixed with ``unix:``
instead of the host, e.g.

```
enabled=True
host=unix:/var/run/redis/redis.sock
```

or

```
enabled = True
instances = nick3@unix:/var/run/redis.sock:/PASSWORD
```

In that case, for disambiguation there must be a colon ``:`` before the slash
``/`` followed by the password.

Note: when using the host/port config mode, the port number is used in
the metric key. When using the multi-instance mode, the nick will be used.
If not specified the port will be used. In case of unix sockets, the base name
without file extension (i.e. in the aforementioned examples ``redis``)
is the default metric key.


"""

import diamond.collector
import time
import os

try:
    import redis
except ImportError:
    redis = None


SOCKET_PREFIX = 'unix:'
SOCKET_PREFIX_LEN = len(SOCKET_PREFIX)


class RedisCollector(diamond.collector.Collector):

    _DATABASE_COUNT = 16
    _DEFAULT_DB = 0
    _DEFAULT_HOST = 'localhost'
    _DEFAULT_PORT = 6379
    _DEFAULT_SOCK_TIMEOUT = 5
    _KEYS = {'clients.blocked': 'blocked_clients',
             'clients.connected': 'connected_clients',
             'clients.longest_output_list': 'client_longest_output_list',
             'cpu.parent.sys': 'used_cpu_sys',
             'cpu.children.sys': 'used_cpu_sys_children',
             'cpu.parent.user': 'used_cpu_user',
             'cpu.children.user': 'used_cpu_user_children',
             'hash_max_zipmap.entries': 'hash_max_zipmap_entries',
             'hash_max_zipmap.value': 'hash_max_zipmap_value',
             'keys.evicted': 'evicted_keys',
             'keys.expired': 'expired_keys',
             'keyspace.hits': 'keyspace_hits',
             'keyspace.misses': 'keyspace_misses',
             'last_save.changes_since': 'changes_since_last_save',
             'last_save.time': 'last_save_time',
             'memory.internal_view': 'used_memory',
             'memory.external_view': 'used_memory_rss',
             'memory.fragmentation_ratio': 'mem_fragmentation_ratio',
             'process.commands_processed': 'total_commands_processed',
             'process.connections_received': 'total_connections_received',
             'process.uptime': 'uptime_in_seconds',
             'pubsub.channels': 'pubsub_channels',
             'pubsub.patterns': 'pubsub_patterns',
             'slaves.connected': 'connected_slaves',
             'slaves.last_io': 'master_last_io_seconds_ago'}
    _RENAMED_KEYS = {'last_save.changes_since': 'rdb_changes_since_last_save',
                     'last_save.time': 'rdb_last_save_time'}

    def process_config(self):
        super(RedisCollector, self).process_config()
        instance_list = self.config['instances']
        # configobj make str of single-element list, let's convert
        if isinstance(instance_list, basestring):
            instance_list = [instance_list]

        # process original single redis instance
        if len(instance_list) == 0:
            host = self.config['host']
            port = int(self.config['port'])
            auth = self.config['auth']
            if auth is not None:
                instance_list.append('%s:%d/%s' % (host, port, auth))
            else:
                instance_list.append('%s:%d' % (host, port))

        self.instances = {}
        for instance in instance_list:

            if '@' in instance:
                (nickname, hostport) = instance.split('@', 1)
            else:
                nickname = None
                hostport = instance

            if hostport.startswith(SOCKET_PREFIX):
                unix_socket, __, port_auth = hostport[
                    SOCKET_PREFIX_LEN:].partition(':')
                auth = port_auth.partition('/')[2] or None

                if nickname is None:
                    nickname = os.path.splitext(
                        os.path.basename(unix_socket))[0]
                self.instances[nickname] = (self._DEFAULT_HOST,
                                            self._DEFAULT_PORT,
                                            unix_socket,
                                            auth)
            else:
                if '/' in hostport:
                    parts = hostport.split('/')
                    hostport = parts[0]
                    auth = parts[1]
                else:
                    auth = None

                if ':' in hostport:
                    if hostport[0] == ':':
                        host = self._DEFAULT_HOST
                        port = int(hostport[1:])
                    else:
                        parts = hostport.split(':')
                        host = parts[0]
                        port = int(parts[1])
                else:
                    host = hostport
                    port = self._DEFAULT_PORT

                if nickname is None:
                    nickname = str(port)

                self.instances[nickname] = (host, port, None, auth)

        self.log.debug("Configured instances: %s" % self.instances.items())

    def get_default_config_help(self):
        config_help = super(RedisCollector, self).get_default_config_help()
        config_help.update({
            'host': 'Hostname to collect from',
            'port': 'Port number to collect from',
            'timeout': 'Socket timeout',
            'db': '',
            'auth': 'Password?',
            'databases': 'how many database instances to collect',
            'instances': "Redis addresses, comma separated, syntax:" +
                         " nick1@host:port, nick2@:port or nick3@host"
        })
        return config_help

    def get_default_config(self):
        """
        Return default config

:rtype: dict

        """
        config = super(RedisCollector, self).get_default_config()
        config.update({
            'host': self._DEFAULT_HOST,
            'port': self._DEFAULT_PORT,
            'timeout': self._DEFAULT_SOCK_TIMEOUT,
            'db': self._DEFAULT_DB,
            'auth': None,
            'databases': self._DATABASE_COUNT,
            'path': 'redis',
            'instances': [],
        })
        return config

    def _client(self, host, port, unix_socket, auth):
        """Return a redis client for the configuration.

:param str host: redis host
:param int port: redis port
:rtype: redis.Redis

        """
        db = int(self.config['db'])
        timeout = int(self.config['timeout'])
        try:
            cli = redis.Redis(host=host, port=port,
                              db=db, socket_timeout=timeout, password=auth,
                              unix_socket_path=unix_socket)
            cli.ping()
            return cli
        except Exception, ex:
            self.log.error("RedisCollector: failed to connect to %s:%i. %s.",
                           unix_socket or host, port, ex)

    def _precision(self, value):
        """Return the precision of the number

:param str value: The value to find the precision of
:rtype: int

        """
        value = str(value)
        decimal = value.rfind('.')
        if decimal == -1:
            return 0
        return len(value) - decimal - 1

    def _publish_key(self, nick, key):
        """Return the full key for the partial key.

:param str nick: Nickname for Redis instance
:param str key: The key name
:rtype: str

        """
        return '%s.%s' % (nick, key)

    def _get_info(self, host, port, unix_socket, auth):
        """Return info dict from specified Redis instance

:param str host: redis host
:param int port: redis port
:rtype: dict

        """

        client = self._client(host, port, unix_socket, auth)
        if client is None:
            return None

        info = client.info()
        del client
        return info

    def _get_config(self, host, port, unix_socket, auth, config_key):
        """Return config string from specified Redis instance and config key

:param str host: redis host
:param int port: redis port
:param str host: redis config_key
:rtype: str

        """

        client = self._client(host, port, unix_socket, auth)
        if client is None:
            return None

        config_value = client.config_get(config_key)
        del client
        return config_value

    def collect_instance(self, nick, host, port, unix_socket, auth):
        """Collect metrics from a single Redis instance

:param str nick: nickname of redis instance
:param str host: redis host
:param int port: redis port
:param str unix_socket: unix socket, if applicable
:param str auth: authentication password

        """

        # Connect to redis and get the info
        info = self._get_info(host, port, unix_socket, auth)
        if info is None:
            return

        # The structure should include the port for multiple instances per
        # server
        data = dict()

        # Connect to redis and get the maxmemory config value
        # Then calculate the % maxmemory of memory used
        maxmemory_config = self._get_config(host, port, unix_socket, auth,
                                            'maxmemory')
        if maxmemory_config and 'maxmemory' in maxmemory_config.keys():
            maxmemory = float(maxmemory_config['maxmemory'])

            # Only report % used if maxmemory is a non zero value
            if maxmemory == 0:
                maxmemory_percent = 0.0
            else:
                maxmemory_percent = info['used_memory'] / maxmemory * 100
                maxmemory_percent = round(maxmemory_percent, 2)
            data['memory.used_percent'] = float("%.2f" % maxmemory_percent)

        # Iterate over the top level keys
        for key in self._KEYS:
            if self._KEYS[key] in info:
                data[key] = info[self._KEYS[key]]

        # Iterate over renamed keys for 2.6 support
        for key in self._RENAMED_KEYS:
            if self._RENAMED_KEYS[key] in info:
                data[key] = info[self._RENAMED_KEYS[key]]

        # Look for databaase speific stats
        for dbnum in range(0, int(self.config.get('databases',
                                                  self._DATABASE_COUNT))):
            db = 'db%i' % dbnum
            if db in info:
                for key in info[db]:
                    data['%s.%s' % (db, key)] = info[db][key]

        # Time since last save
        for key in ['last_save_time', 'rdb_last_save_time']:
            if key in info:
                data['last_save.time_since'] = int(time.time()) - info[key]

        # Publish the data to graphite
        for key in data:
            self.publish(self._publish_key(nick, key),
                         data[key],
                         precision=self._precision(data[key]),
                         metric_type='GAUGE')

    def collect(self):
        """Collect the stats from the redis instance and publish them.

        """
        if redis is None:
            self.log.error('Unable to import module redis')
            return {}

        for nick in self.instances.keys():
            (host, port, unix_socket, auth) = self.instances[nick]
            self.collect_instance(nick, host, int(port), unix_socket, auth)



# coding=utf-8

"""
Collecting connections tracking statistics from nf_conntrack/ip_conntrack
kernel module.

#### Dependencies

 * nf_conntrack/ip_conntrack kernel module

"""

import diamond.collector
import os


class ConnTrackCollector(diamond.collector.Collector):
    """
    Collector of number of conntrack connections
    """

    def get_default_config_help(self):
        """
        Return help text for collector configuration
        """
        config_help = super(ConnTrackCollector, self).get_default_config_help()
        config_help.update({
            "dir":      "Directories with files of interest, comma seperated",
            "files":    "List of files to collect statistics from",
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(ConnTrackCollector, self).get_default_config()
        config.update({
            "path":  "conntrack",
            "dir":   "/proc/sys/net/ipv4/netfilter,/proc/sys/net/netfilter",
            "files": "ip_conntrack_count,ip_conntrack_max,"
                     "nf_conntrack_count,nf_conntrack_max",
        })
        return config

    def collect(self):
        """
        Collect metrics
        """
        collected = {}
        files = []

        if isinstance(self.config['dir'], basestring):
            dirs = [d.strip() for d in self.config['dir'].split(',')]
        elif isinstance(self.config['dir'], list):
            dirs = self.config['dir']

        if isinstance(self.config['files'], basestring):
            files = [f.strip() for f in self.config['files'].split(',')]
        elif isinstance(self.config['files'], list):
            files = self.config['files']

        for sdir in dirs:
            for sfile in files:
                if sfile.endswith('conntrack_count'):
                    metric_name = 'ip_conntrack_count'
                elif sfile.endswith('conntrack_max'):
                    metric_name = 'ip_conntrack_max'
                else:
                    self.log.error('Unknown file for collection: %s', sfile)
                    continue
                fpath = os.path.join(sdir, sfile)
                if not os.path.exists(fpath):
                    continue
                try:
                    with open(fpath, "r") as fhandle:
                        metric = float(fhandle.readline().rstrip("\n"))
                        collected[metric_name] = metric
                except Exception as exception:
                    self.log.error("Failed to collect from '%s': %s",
                                   fpath,
                                   exception)
        if not collected:
            self.log.error('No metric was collected, looks like '
                           'nf_conntrack/ip_conntrack kernel module was '
                           'not loaded')
        else:
            for key in collected.keys():
                self.publish(key, collected[key])



# coding=utf-8

"""
Collect counters from Netfilter accounting

#### Dependencies

 * [nfacct](http://www.netfilter.org/projects/nfacct/)

"""

import diamond.collector
from subprocess import Popen, PIPE
import re

from diamond.collector import str_to_bool


class NetfilterAccountingCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = (
            super(NetfilterAccountingCollector, self).get_default_config_help())
        config_help.update({
            'bin': 'The path to the smartctl binary',
            'reset': 'Reset counters after collecting',
            'use_sudo': 'Use sudo?',
            'sudo_cmd': 'Path to sudo',
        })
        return config_help

    def get_default_config(self):
        """
        Returns default configuration options.
        """
        config = super(NetfilterAccountingCollector, self).get_default_config()
        config.update({
            'path': 'nfacct',
            'bin': 'nfacct',
            'use_sudo': False,
            'reset': True,
            'sudo_cmd': '/usr/bin/sudo',
            'method': 'Threaded'
        })
        return config

    def collect(self):
        """
        Collect and publish netfilter counters
        """
        cmd = [self.config['bin'], "list"]

        if str_to_bool(self.config['reset']):
            cmd.append("reset")

        if str_to_bool(self.config['use_sudo']):
            cmd.insert(0, self.config['sudo_cmd'])

        # We avoid use of the XML format to mtaintain compatbility with older
        # versions of nfacct and also to avoid the bug where pkts and bytes were
        # flipped

        # Each line is of the format:
        # { pkts = 00000000000001121700, bytes = 00000000000587037355 } = ipv4;
        matcher = re.compile("{ pkts = (.*), bytes = (.*) } = (.*);")
        lines = Popen(cmd, stdout=PIPE).communicate()[0].strip().splitlines()

        for line in lines:
            matches = re.match(matcher, line)
            if matches:
                num_packets = int(matches.group(1))
                num_bytes = int(matches.group(2))
                name = matches.group(3)
                self.publish(name + ".pkts", num_packets)
                self.publish(name + ".bytes", num_bytes)



# coding=utf-8

"""
Collects data from php-fpm if the pm.status_path is enabled


#### Usage

A sample php-fpm config for this collector to work is

```
pm.status_path = /fpm-status
```

#### Dependencies

 * urllib2
 * json (or simeplejson)

"""

try:
    import json
except ImportError:
    import simplejson as json

import urllib2
import diamond.collector


class PhpFpmCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(PhpFpmCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(PhpFpmCollector, self).get_default_config()
        config.update({
            'host': 'localhost',
            'port': 80,
            'uri': 'fpm-status',
            'byte_unit': ['byte'],
            'path': 'phpfpm',
        })
        return config

    def collect(self):
        #
        # if there is a / in front remove it
        if self.config['uri'][0] == '/':
            self.config['uri'] = self.config['uri'][1:]

        try:
            response = urllib2.urlopen("http://%s:%s/%s?json" % (
                self.config['host'], int(self.config['port']),
                self.config['uri']))
        except Exception, e:
            self.log.error('Couldnt connect to php-fpm status page: %s', e)
            return {}

        try:
            j = json.loads(response.read())
        except Exception, e:
            self.log.error('Couldnt parse json: %s', e)
            return {}

        valid_metrics = [
            'accepted_conn',
            'listen_queue',
            'max_listen_queue',
            'listen_queue_len',
            'idle_processes',
            'active_processes',
            'total_processes',
            'max_active_processes',
            'max_children_reached',
            'slow_requests'
        ]
        for k, v in j.items():
            #
            # php-fpm has spaces in the keys so lets replace all spaces with _
            k = k.replace(" ", "_")

            if k in valid_metrics:
                self.publish(k, v)



# coding=utf-8

"""
The TCPCollector class collects metrics on TCP stats

#### Dependencies

 * /proc/net/netstat
 * /proc/net/snmp

#### Allowed Metric Names
<table>
<tr><th>Name</th><th>Description</th></tr>
<tr><td>SyncookiesSent</td><td>An application wasn't able to accept a
connection fast enough, so the kernel couldn't store an entry in the queue for
this connection. Instead of dropping it, it sent a cookie to the client.
</td></tr>
<tr><td>SyncookiesRecv</td><td>After sending a cookie, it came back to us
and passed the check.</td></tr>
<tr><td>SyncookiesFailed</td><td>After sending a cookie, it came back to us
but looked invalid.</td></tr>
<tr><td>EmbryonicRsts</td><td></td></tr>
<tr><td>PruneCalled</td><td></td></tr>
<tr><td>RcvPruned</td><td>If the kernel is really really desperate and cannot
give more memory to this socket even after dropping the ofo queue, it will
simply discard the packet it received.  This is Really Bad.</td></tr>
<tr><td>OfoPruned</td><td>When a socket is using too much memory (rmem), the
kernel will first discard any out-of-order packet that has been queued (with
SACK).</td></tr>
<tr><td>OutOfWindowIcmps</td><td></td></tr>
<tr><td>LockDroppedIcmps</td><td></td></tr>
<tr><td>ArpFilter</td><td></td></tr>
<tr><td>TW</td><td></td></tr>
<tr><td>TWRecycled</td><td></td></tr>
<tr><td>TWKilled</td><td></td></tr>
<tr><td>PAWSPassive</td><td></td></tr>
<tr><td>PAWSActive</td><td></td></tr>
<tr><td>PAWSEstab</td><td></td></tr>
<tr><td>DelayedACKs</td><td>We waited for another packet to send an ACK, but
didn't see any, so a timer ended up sending a delayed ACK.</td></tr>
<tr><td>DelayedACKLocked</td><td>We wanted to send a delayed ACK but failed
because the socket was locked.  So the timer was reset.</td></tr>
<tr><td>DelayedACKLost</td><td>We sent a delayed and duplicated ACK because the
remote peer retransmitted a packet, thinking that it didn't get to us.</td></tr>
<tr><td>ListenOverflows</td><td>We completed a 3WHS but couldn't put the socket
on the accept queue, so we had to discard the connection.</td></tr>
<tr><td>ListenDrops</td><td>We couldn't accept a connection because one of: we
had no route to the destination, we failed to allocate a socket, we failed to
allocate a new local port bind bucket.  Note: this counter also include all the
increments made to ListenOverflows</td></tr>
<tr><td>TCPPrequeued</td><td></td></tr>
<tr><td>TCPDirectCopyFromBacklog</td><td></td></tr>
<tr><td>TCPDirectCopyFromPrequeue</td><td></td></tr>
<tr><td>TCPPrequeueDropped</td><td></td></tr>
<tr><td>TCPHPHits</td><td></td></tr>
<tr><td>TCPHPHitsToUser</td><td></td></tr>
<tr><td>TCPPureAcks</td><td></td></tr>
<tr><td>TCPHPAcks</td><td></td></tr>
<tr><td>TCPRenoRecovery</td><td>A packet was lost and we recovered after a
fast retransmit.</td></tr>
<tr><td>TCPSackRecovery</td><td>A packet was lost and we recovered by using
selective acknowledgements.</td></tr>
<tr><td>TCPSACKReneging</td><td></td></tr>
<tr><td>TCPFACKReorder</td><td>We detected re-ordering using FACK (Forward ACK
-- the highest sequence number known to have been received by the peer when
using SACK -- FACK is used during congestion control).</td></tr>
<tr><td>TCPSACKReorder</td><td>We detected re-ordering using SACK.</td></tr>
<tr><td>TCPRenoReorder</td><td>We detected re-ordering using fast retransmit.
</td></tr>
<tr><td>TCPTSReorder</td><td>We detected re-ordering using the timestamp option.
</td></tr>
<tr><td>TCPFullUndo</td><td>We detected some erroneous retransmits and undid our
CWND reduction.</td></tr>
<tr><td>TCPPartialUndo</td><td>We detected some erroneous retransmits, a partial
ACK arrived while we were fast retransmitting, so we were able to partially undo
some of our CWND reduction.</td></tr>
<tr><td>TCPDSACKUndo</td><td>We detected some erroneous retransmits, a D-SACK
arrived and ACK'ed all the retransmitted data, so we undid our CWND reduction.
</td></tr>
<tr><td>TCPLossUndo</td><td>We detected some erroneous retransmits, a partial
ACK arrived, so we undid our CWND reduction.</td></tr>
<tr><td>TCPLoss</td><td></td></tr>
<tr><td>TCPLostRetransmit</td><td></td></tr>
<tr><td>TCPRenoFailures</td><td></td></tr>
<tr><td>TCPSackFailures</td><td></td></tr>
<tr><td>TCPLossFailures</td><td></td></tr>
<tr><td>TCPFastRetrans</td><td></td></tr>
<tr><td>TCPForwardRetrans</td><td></td></tr>
<tr><td>TCPSlowStartRetrans</td><td></td></tr>
<tr><td>TCPTimeouts</td><td></td></tr>
<tr><td>TCPRenoRecoveryFail</td><td></td></tr>
<tr><td>TCPSackRecoveryFail</td><td></td></tr>
<tr><td>TCPSchedulerFailed</td><td></td></tr>
<tr><td>TCPRcvCollapsed</td><td></td></tr>
<tr><td>TCPDSACKOldSent</td><td></td></tr>
<tr><td>TCPDSACKOfoSent</td><td></td></tr>
<tr><td>TCPDSACKRecv</td><td></td></tr>
<tr><td>TCPDSACKOfoRecv</td><td></td></tr>

<tr><td>TCPSACKDiscard</td><td>We got a completely invalid SACK block and
discarded it.</td></tr>
<tr><td>TCPDSACKIgnoredOld</td><td>We got a duplicate SACK while retransmitting
so we discarded it.</td></tr>
<tr><td>TCPDSACKIgnoredNoUndo</td><td>We got a duplicate SACK and discarded it.
</td></tr>

<tr><td>TCPAbortOnSyn</td><td>We received an unexpected SYN so we sent a RST to
the peer.</td></tr>
<tr><td>TCPAbortOnData</td><td>We were in FIN_WAIT1 yet we received a data
packet with a sequence number that's beyond the last one for this connection,
so we RST'ed.</td></tr>
<tr><td>TCPAbortOnClose</td><td>We received data but the user has closed the
socket, so we have no wait of handing it to them, so we RST'ed.</td></tr>
<tr><td>TCPAbortOnMemory</td><td>This is Really Bad.  It happens when there are
too many orphaned sockets (not attached a FD) and the kernel has to drop a
connection. Sometimes it will send a reset to the peer, sometimes it wont.
</td></tr>
<tr><td>TCPAbortOnTimeout</td><td>The connection timed out really hard.
</td></tr>
<tr><td>TCPAbortOnLinger</td><td>We killed a socket that was closed by the
application and lingered around for long enough.</td></tr>
<tr><td>TCPAbortFailed</td><td>We tried to send a reset, probably during one of
teh TCPABort* situations above, but we failed e.g. because we couldn't allocate
enough memory (very bad).</td></tr>
<tr><td>TCPMemoryPressures</td><td>Number of times a socket was put in "memory
pressure" due to a non fatal memory allocation failure (reduces the send buffer
size etc).</td></tr>

<tr><td>TCPBacklogDrop</td><td>We received something but had to drop it because
the socket's receive queue was full.</td></tr>

<tr><td>RtoAlgorithm</td><td>The algorithm used to determine the timeout value
used for retransmitting unacknowledged octets.</td></tr>
<tr><td>RtoMin</td><td>The minimum value permitted by a TCP implementation
for the retransmission timeout, measured in milliseconds.
More refined semantics for objects of this type depend upon the algorithm used
to determine the retransmission timeout. In particular,
when the timeout algorithm is ``rsre '' (3), an object of this type has the
semantics of the LBOUND quantity described in RFC 793.</td></tr>
<tr><td>RtoMax</td><td>The maximum value permitted by a TCP implementation for
the retransmission timeout, measured in milliseconds. More refined semantics
for objects of this type depend upon the algorithm used to determine the
retransmission timeout. In particular, when the timeout algorithm is ``rsre''
(3), an object of this type has the semantics of the UBOUND quantity described
in RFC 793.</td></tr>
<tr><td>MaxConn</td><td>The limit on the total number of TCP connections the
entity can support. In entities where the maximum number of connections is
dynamic, this object should contain the value -1.</td></tr>
<tr><td>ActiveOpens</td><td>The number of times TCP connections have made a
direct transition to the SYN-SENT state from the CLOSED state.</td></tr>
<tr><td>PassiveOpens</td><td>The number of times TCP connections have made a
direct transition to the SYN-RCVD state from the LISTEN state.</td></tr>
<tr><td>AttemptFails</td><td>The number of times TCP connections have made a
direct transition to the CLOSED state from either the SYN-SENT state or the
SYN-RCVD state, plus the number of times TCP connections have made a direct
transition to the LISTEN state from the SYN-RCVD state.</td></tr>
<tr><td>EstabResets</td><td>The number of times TCP connections have made a
direct transition to the CLOSED state from either the ESTABLISHED state or the
CLOSE-WAIT state.</td></tr>
<tr><td>CurrEstab</td><td>The number of TCP connections for which the current
state is either ESTABLISHED or CLOSE- WAIT.</td></tr>
<tr><td>InSegs</td><td>The total number of segments received, including those
received in error. This count includes segments received on currently
established connections.</td></tr>
<tr><td>OutSegs</td><td>The total number of segments sent, including those on
current connections but excluding those containing only retransmitted octets.
</td></tr>
<tr><td>RetransSegs</td><td>The total number of segments retransmitted - that
is, the number of TCP segments transmitted containing one or more previously
transmitted octets.</td></tr>
<tr><td>InErrs</td><td>The total number of segments received in error (for
example, bad TCP checksums).</td></tr>
<tr><td>OutRsts</td><td>The number of TCP segments sent containing the RST
flag.</td></tr>
</table>

"""

import diamond.collector
import os


class TCPCollector(diamond.collector.Collector):

    PROC = [
        '/proc/net/netstat',
        '/proc/net/snmp'
    ]

    def process_config(self):
        super(TCPCollector, self).process_config()
        if self.config['allowed_names'] is None:
            self.config['allowed_names'] = []

        if self.config['gauges'] is None:
            self.config['gauges'] = ['CurrEstab', 'MaxConn']

    def get_default_config_help(self):
        config_help = super(TCPCollector, self).get_default_config_help()
        config_help.update({
            'allowed_names': 'list of entries to collect, empty to collect all',
            'gauges': 'list of metrics to be published as gauges',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(TCPCollector, self).get_default_config()
        config.update({
            'path': 'tcp',
            'allowed_names':
                'ListenOverflows, ListenDrops, TCPLoss, ' +
                'TCPTimeouts, TCPFastRetrans, TCPLostRetransmit, ' +
                'TCPForwardRetrans, TCPSlowStartRetrans, CurrEstab, ' +
                'TCPAbortOnMemory, TCPBacklogDrop, AttemptFails, ' +
                'EstabResets, InErrs, ActiveOpens, PassiveOpens',
            'gauges': 'CurrEstab, MaxConn',
        })
        return config

    def collect(self):
        metrics = {}

        for filepath in self.PROC:
            if not os.access(filepath, os.R_OK):
                self.log.error('Permission to access %s denied', filepath)
                continue

            header = ''
            data = ''

            # Seek the file for the lines that start with Tcp
            file = open(filepath)

            if not file:
                self.log.error('Failed to open %s', filepath)
                continue

            while True:
                line = file.readline()

                # Reached EOF?
                if len(line) == 0:
                    break

                # Line has metrics?
                if line.startswith("Tcp"):
                    header = line
                    data = file.readline()
                    break
            file.close()

            # No data from the file?
            if header == '' or data == '':
                self.log.error('%s has no lines with Tcp', filepath)
                continue

            header = header.split()
            data = data.split()

            for i in xrange(1, len(header)):
                metrics[header[i]] = data[i]

        for metric_name in metrics.keys():
            if ((len(self.config['allowed_names']) > 0 and
                 metric_name not in self.config['allowed_names'])):
                continue

            value = long(metrics[metric_name])

            # Publish the metric
            if metric_name in self.config['gauges']:
                self.publish_gauge(metric_name, value, 0)
            else:
                self.publish_counter(metric_name, value, 0)



# coding=utf-8
"""
Gather HTTP Response code and Duration of HTTP request

#### Dependencies
  * urllib2

"""

import urllib2
import time
from datetime import datetime
import diamond.collector


class WebsiteMonitorCollector(diamond.collector.Collector):
    """
    Gather HTTP response code and Duration of HTTP request
    """

    def get_default_config_help(self):
        config_help = super(WebsiteMonitorCollector,
                            self).get_default_config_help()
        config_help.update({
            'URL': "FQDN of HTTP endpoint to test",

        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        default_config = super(WebsiteMonitorCollector,
                               self).get_default_config()
        default_config['URL'] = ''
        default_config['path'] = 'websitemonitor'
        return default_config

    def collect(self):
        req = urllib2.Request('%s' % (self.config['URL']))

        try:
            # time in seconds since epoch as a floating number
            start_time = time.time()
            # human-readable time e.g November 25, 2013 18:15:56
            st = datetime.fromtimestamp(start_time
                                        ).strftime('%B %d, %Y %H:%M:%S')
            self.log.debug('Start time: %s' % (st))

            resp = urllib2.urlopen(req)
            # time in seconds since epoch as a floating number
            end_time = time.time()
            # human-readable end time e.eg. November 25, 2013 18:15:56
            et = datetime.fromtimestamp(end_time).strftime('%B %d, %Y %H:%M%S')
            self.log.debug('End time: %s' % (et))
            # Response time in milliseconds
            rt = int(format((end_time - start_time) * 1000, '.0f'))
            # Publish metrics
            self.publish('response_time.%s' % (resp.code), rt,
                         metric_type='COUNTER')
        # urllib2 will puke on non HTTP 200/OK URLs
        except urllib2.URLError, e:
            if e.code != 200:
                # time in seconds since epoch as a floating number
                end_time = time.time()
                # Response time in milliseconds
                rt = int(format((end_time - start_time) * 1000, '.0f'))
                # Publish metrics -- this is recording a failure, rt will
                # likely be 0 but does capture HTTP Status Code
                self.publish('response_time.%s' % (e.code), rt,
                             metric_type='COUNTER')

        except IOError, e:
            self.log.error('Unable to open %s' % (self.config['URL']))

        except Exception, e:
            self.log.error("Unknown error opening url: %s", e)



# coding=utf-8

import time
import math
import multiprocessing
import os
import random
import sys
import signal

try:
    from setproctitle import getproctitle, setproctitle
except ImportError:
    setproctitle = None

from diamond.utils.signals import signal_to_exception
from diamond.utils.signals import SIGALRMException
from diamond.utils.signals import SIGHUPException


def collector_process(collector, metric_queue, log):
    """
    """
    proc = multiprocessing.current_process()
    if setproctitle:
        setproctitle('%s - %s' % (getproctitle(), proc.name))

    signal.signal(signal.SIGALRM, signal_to_exception)
    signal.signal(signal.SIGHUP, signal_to_exception)
    signal.signal(signal.SIGUSR2, signal_to_exception)

    interval = float(collector.config['interval'])

    log.debug('Starting')
    log.debug('Interval: %s seconds', interval)

    # Validate the interval
    if interval <= 0:
        log.critical('interval of %s is not valid!', interval)
        sys.exit(1)

    # Start the next execution at the next window plus some stagger delay to
    # avoid having all collectors running at the same time
    next_window = math.floor(time.time() / interval) * interval
    stagger_offset = random.uniform(0, interval - 1)

    # Allocate time till the end of the window for the collector to run. With a
    # minimum of 1 second
    max_time = int(max(interval - stagger_offset, 1))
    log.debug('Max collection time: %s seconds', max_time)

    # Setup stderr/stdout as /dev/null so random print statements in thrid
    # party libs do not fail and prevent collectors from running.
    # https://github.com/BrightcoveOS/Diamond/issues/722
    sys.stdout = open(os.devnull, 'w')
    sys.stderr = open(os.devnull, 'w')

    while(True):
        try:
            time_to_sleep = (next_window + stagger_offset) - time.time()
            if time_to_sleep > 0:
                time.sleep(time_to_sleep)
            elif time_to_sleep < 0:
                # clock has jumped, lets skip missed intervals
                next_window = time.time()

            next_window += interval

            # Ensure collector run times fit into the collection window
            signal.alarm(max_time)

            # Collect!
            collector._run()

            # Success! Disable the alarm
            signal.alarm(0)

        except SIGALRMException:
            log.error('Took too long to run! Killed!')

            # Adjust  the stagger_offset to allow for more time to run the
            # collector
            stagger_offset = stagger_offset * 0.9

            max_time = int(max(interval - stagger_offset, 1))
            log.debug('Max collection time: %s seconds', max_time)

        except SIGHUPException:
            # Reload the config if requested
            # We must first disable the alarm as we don't want it to interrupt
            # us and end up with half a loaded config
            signal.alarm(0)

            log.info('Reloading config reload due to HUP')
            collector.load_config()
            log.info('Config reloaded')

        except Exception:
            log.exception('Collector failed!')
            break


def handler_process(handlers, metric_queue, log):
    proc = multiprocessing.current_process()
    if setproctitle:
        setproctitle('%s - %s' % (getproctitle(), proc.name))

    log.debug('Starting process %s', proc.name)

    while(True):
        metric = metric_queue.get(block=True, timeout=None)
        for handler in handlers:
            if metric is not None:
                handler._process(metric)
            else:
                handler._flush()



# coding=utf-8

"""
The XENCollector grabs usage/allocation metrics using libvirt

#### Dependencies
 * python-libvirt

"""

from diamond.collector import Collector

import os
try:
    import libvirt
except ImportError:
    libvirt = None


class XENCollector(Collector):

    def get_default_config_help(self):
        config_help = super(XENCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(XENCollector, self).get_default_config()
        config.update({
            'path':     'xen'
        })
        return config

    def collect(self):
        """
        Collect libvirt data
        """
        if libvirt is None:
            self.log.error('Unable to import either libvirt')
            return {}
        # Open a restricted (non-root) connection to the hypervisor
        conn = libvirt.openReadOnly(None)
        # Get hardware info
        conninfo = conn.getInfo()
        # Initialize variables
        memallocated = 0
        coresallocated = 0
        totalcores = 0
        results = {}
        domIds = conn.listDomainsID()
        if 0 in domIds:
            # Total cores
            domU = conn.lookupByID(0)
            totalcores = domU.info()[3]
        # Free Space
        s = os.statvfs('/')
        freeSpace = (s.f_bavail * s.f_frsize) / 1024
        # Calculate allocated memory and cores
        for i in domIds:
            # Ignore 0
            if i == 0:
                continue
            domU = conn.lookupByID(i)
            dominfo = domU.info()
            memallocated += dominfo[2]
            if i > 0:
                coresallocated += dominfo[3]
        results = {
            'InstalledMem': conninfo[1],
            'MemAllocated': memallocated / 1024,
            'MemFree': conninfo[1] - (memallocated / 1024),
            'AllocatedCores': coresallocated,
            'DiskFree': freeSpace,
            'TotalCores': totalcores,
            'FreeCores': (totalcores - coresallocated)
        }
        for k in results.keys():
            self.publish(k, results[k], 0)



# coding=utf-8

"""
Collect counters from scribe

#### Dependencies

    * /usr/sbin/scribe_ctrl, distributed with scribe

"""

import subprocess
import string

import diamond.collector


class ScribeCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(ScribeCollector, self).get_default_config_help()
        config_help.update({
            'scribe_ctrl_bin': 'Path to scribe_ctrl binary',
            'scribe_port': 'Scribe port',
        })
        return config_help

    def get_default_config(self):
        config = super(ScribeCollector, self).get_default_config()
        config.update({
            'path': 'scribe',
            'scribe_ctrl_bin': self.find_binary('/usr/sbin/scribe_ctrl'),
            'scribe_port': None,
        })
        return config

    def key_to_metric(self, key):
        """Replace all non-letter characters with underscores"""
        return ''.join(l if l in string.letters else '_' for l in key)

    def get_scribe_ctrl_output(self):
        cmd = [self.config['scribe_ctrl_bin'], 'counters']

        if self.config['scribe_port'] is not None:
            cmd.append(self.config['scribe_port'])

        self.log.debug("Running command %r", cmd)

        try:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
        except OSError:
            self.log.exception("Unable to run %r", cmd)
            return ""

        stdout, stderr = p.communicate()

        if p.wait() != 0:
            self.log.warning("Command failed %r", cmd)
            self.log.warning(stderr)

        return stdout

    def get_scribe_stats(self):
        output = self.get_scribe_ctrl_output()

        data = {}

        for line in output.splitlines():
            key, val = line.rsplit(':', 1)
            metric = self.key_to_metric(key)
            data[metric] = int(val)

        return data.items()

    def collect(self):
        for stat, val in self.get_scribe_stats():
            self.publish(stat, val)



# coding=utf-8

"""
A Simple collector which calls the Eventstore API for
the status of projections and parses this to flat metrics.
String values are ignored, except for the name and status.
Name is used for the metric path, status is translated to
an integer (running = 1, stopped = 0).
Note: "$" are replaced by underscores (_) in the projection
name to avoid problems with hostedGraphite/Grafana.

This collector is based upon the HTTPJSONCollector.

#### Dependencies

 * urllib2

"""

import urllib2
import json
import diamond.collector


class EventstoreProjectionsCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(
            EventstoreProjectionsCollector, self).get_default_config_help(
        )
        config_help.update({
            'path': "name of the metric in the metricpath",
            'protocol': 'protocol used to connect to eventstore',
            'hostname': 'hostname of the eventstore instance',
            'route': 'route in eventstore for projections',
            'port': 'tcp port where eventstore is listening',
            'headers': 'Header variable if needed',
            'replace_dollarsign':
            'A value to replace a dollar sign ($) in projection names by',
            'debug': 'Enable or disable debug mode',
        })
        return config_help

    def get_default_config(self):
        default_config = super(
            EventstoreProjectionsCollector, self).get_default_config(
        )
        default_config.update({
            'path': "eventstore",
            'protocol': 'http://',
            'hostname': 'localhost',
            'route': '/projections/all-non-transient',
            'port': 2113,
            'headers': {'User-Agent': 'Diamond Eventstore metrics collector'},
            'replace_dollarsign': '_',
            'debug': False,
        })
        return default_config

    def _json_to_flat_metrics(self, prefix, data):

        for key, value in data.items():
            if isinstance(value, dict):
                for k, v in self._json_to_flat_metrics(
                        "%s.%s" % (prefix, key), value):
                    yield k, v
            elif isinstance(value, basestring):
                if value == "Running":
                    value = 1
                    yield ("%s.%s" % (prefix, key), value)
                elif value == "Stopped":
                    value = 0
                    yield ("%s.%s" % (prefix, key), value)
                else:
                    if self.config['debug']:
                        self.log.debug("ignoring string value = %s", value)
                    continue
            else:
                try:
                    int(value)
                except ValueError:
                    self.log.debug("cast to int failed, value = %s", value)
                finally:
                    yield ("%s.%s" % (prefix, key), value)

    def collect(self):
        eventstore_host = "%s%s:%s%s" % (
            self.config['protocol'],
            self.config['hostname'],
            self.config['port'],
            self.config['route']
        )

        req = urllib2.Request(eventstore_host, headers=self.config['headers'])
        req.add_header('Content-type', 'application/json')

        try:
            resp = urllib2.urlopen(req)
        except urllib2.URLError as e:
            self.log.error("Can't open url %s. %s", eventstore_host, e)
        else:
            content = resp.read()
            try:
                json_dict = json.loads(content)
                projections = json_dict['projections']

                data = {}
                for projection in projections:
                    if self.config['replace_dollarsign']:
                        name = projection["name"].replace(
                            '$',
                            self.config['replace_dollarsign']
                        )
                    else:
                        name = projection["name"]
                    data[name] = projection
            except ValueError as e:
                self.log.error("failed parsing JSON Object \
                                from %s. %s", eventstore_host, e)
            else:
                for metric_name, metric_value in self._json_to_flat_metrics(
                        "projections", data):
                    self.publish(metric_name, metric_value)



# coding=utf-8

"""
SNMPCollector for Fusion IO DRives Metrics. ( Subclass of snmpCollector )
Based heavily on the NetscalerSNMPCollector.

This collector currently assumes a single IODrive I or IODrive II and not the
DUO, Octals, or multiple IODrive I or IIs. It needs to be enhanced to account
for multiple fio devices. ( Donations being accepted )

The metric path is configured to be under servers.<host>.<device> where host
and device is defined in the IODriveSNMPCollector.conf.  So given the example
conf below the metricpath would be
"servers.my_host.iodrive.<metric> name.

# EXAMPLE CONF file

enabled = True
[devices]
[[iodrive]]
host = my_host
port = 161
community = mycommunitystring

"""

import sys
import os
import time
import struct

# Fix Path for locating the SNMPCollector
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             '../',
                                             'snmp',
                                             )))

from diamond.metric import Metric
from snmp import SNMPCollector as parent_SNMPCollector


class IODriveSNMPCollector(parent_SNMPCollector):
    """
    SNMPCollector for a single Fusion IO Drive
    """

    IODRIVE_STATS = {

        "InternalTemp": "1.3.6.1.4.1.30018.1.2.1.1.1.24.5",

        "MilliVolts": "1.3.6.1.4.1.30018.1.2.1.1.1.32.5",
        "MilliWatts": "1.3.6.1.4.1.30018.1.2.1.1.1.35.5",
        "MilliAmps": "1.3.6.1.4.1.30018.1.2.1.1.1.37.5",
    }

    IODRIVE_BYTE_STATS = {

        "BytesReadU": "1.3.6.1.4.1.30018.1.2.2.1.1.12.5",
        "BytesReadL": "1.3.6.1.4.1.30018.1.2.2.1.1.13.5",

        "BytesWrittenU": "1.3.6.1.4.1.30018.1.2.2.1.1.14.5",
        "BytesWrittenL": "1.3.6.1.4.1.30018.1.2.2.1.1.15.5",

    }

    MAX_VALUE = 18446744073709551615

    def get_default_config_help(self):
        config_help = super(IODriveSNMPCollector,
                            self).get_default_config_help()
        config_help.update({
            'host': 'Host address',
            'port': 'SNMP port to collect snmp data',
            'community': 'SNMP community',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(IODriveSNMPCollector, self).get_default_config()
        config.update({
            'path':     'iodrive',
            'timeout':  15,
        })
        return config

    def get_string_index_oid(self, s):
        """Turns a string into an oid format is length of name followed by
        name chars in ascii"""
        return (len(self.get_bytes(s)), ) + self.get_bytes(s)

    def get_bytes(self, s):
        """Turns a string into a list of byte values"""
        return struct.unpack('%sB' % len(s), s)

    def collect_snmp(self, device, host, port, community):
        """
        Collect Fusion IO Drive SNMP stats from device
        host and device are from the conf file. In the future device should be
        changed to be what IODRive device it being checked.
        i.e. fioa, fiob.
        """

        # Set timestamp
        timestamp = time.time()

        for k, v in self.IODRIVE_STATS.items():
            # Get Metric Name and Value
            metricName = '.'.join([k])
            metricValue = int(self.get(v, host, port, community)[v])

            # Get Metric Path
            metricPath = '.'.join(['servers', host, device, metricName])

            # Create Metric
            metric = Metric(metricPath, metricValue, timestamp, 0)

            # Publish Metric
            self.publish_metric(metric)

        for k, v in self.IODRIVE_BYTE_STATS.items():
            # Get Metric Name and Value
            metricName = '.'.join([k])
            metricValue = int(self.get(v, host, port, community)[v])

            # Get Metric Path
            metricPath = '.'.join(['servers', host, device, metricName])

            # Create Metric
            metric = Metric(metricPath, metricValue, timestamp, 0)

            # Publish Metric
            self.publish_metric(metric)



# coding=utf-8

"""
Send metrics to a [influxdb](https://github.com/influxdb/influxdb/) using the
http interface.

v1.0 : creation
v1.1 : force influxdb driver with SSL
v1.2 : added a timer to delay influxdb writing in case of failure
       this whill avoid the 100% cpu loop when influx in not responding
       Sebastien Prune THOMAS - prune@lecentre.net

#### Dependencies
 * [influxdb](https://github.com/influxdb/influxdb-python)


#### Configuration
```
[[InfluxdbHandler]]
hostname = localhost
port = 8086 #8084 for HTTPS
batch_size = 100 # default to 1
cache_size = 1000 # default to 20000
username = root
password = root
database = graphite
time_precision = s
```
"""

import time
from Handler import Handler

try:
    from influxdb.client import InfluxDBClient
except ImportError:
    InfluxDBClient = None


class InfluxdbHandler(Handler):
    """
    Sending data to Influxdb using batched format
    """

    def __init__(self, config=None):
        """
        Create a new instance of the InfluxdbeHandler
        """
        # Initialize Handler
        Handler.__init__(self, config)

        if not InfluxDBClient:
            self.log.error('influxdb.client.InfluxDBClient import failed. '
                           'Handler disabled')
            self.enabled = False
            return

        # Initialize Options
        if self.config['ssl'] == "True":
            self.ssl = True
        else:
            self.ssl = False
        self.hostname = self.config['hostname']
        self.port = int(self.config['port'])
        self.username = self.config['username']
        self.password = self.config['password']
        self.database = self.config['database']
        self.batch_size = int(self.config['batch_size'])
        self.metric_max_cache = int(self.config['cache_size'])
        self.batch_count = 0
        self.time_precision = self.config['time_precision']

        # Initialize Data
        self.batch = {}
        self.influx = None
        self.batch_timestamp = time.time()
        self.time_multiplier = 1

        # Connect
        self._connect()

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(InfluxdbHandler, self).get_default_config_help()

        config.update({
            'hostname': 'Hostname',
            'port': 'Port',
            'ssl': 'set to True to use HTTPS instead of http',
            'batch_size': 'How many metrics to store before sending to the'
            ' influxdb server',
            'cache_size': 'How many values to store in cache in case of'
            ' influxdb failure',
            'username': 'Username for connection',
            'password': 'Password for connection',
            'database': 'Database name',
            'time_precision': 'time precision in second(s), milisecond(ms) or '
            'microsecond (u)',
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(InfluxdbHandler, self).get_default_config()

        config.update({
            'hostname': 'localhost',
            'port': 8086,
            'ssl': False,
            'username': 'root',
            'password': 'root',
            'database': 'graphite',
            'batch_size': 1,
            'cache_size': 20000,
            'time_precision': 's',
        })

        return config

    def __del__(self):
        """
        Destroy instance of the InfluxdbHandler class
        """
        self._close()

    def process(self, metric):
        if self.batch_count <= self.metric_max_cache:
            # Add the data to the batch
            self.batch.setdefault(metric.path, []).append([metric.timestamp,
                                                           metric.value])
            self.batch_count += 1
        # If there are sufficient metrics, then pickle and send
        if self.batch_count >= self.batch_size and (
                time.time() - self.batch_timestamp) > 2**self.time_multiplier:
            # Log
            self.log.debug(
                "InfluxdbHandler: Sending batch sizeof : %d/%d after %fs",
                self.batch_count,
                self.batch_size,
                (time.time() - self.batch_timestamp))
            # reset the batch timer
            self.batch_timestamp = time.time()
            # Send pickled batch
            self._send()
        else:
            self.log.debug(
                "InfluxdbHandler: not sending batch of %d as timestamp is %f",
                self.batch_count,
                (time.time() - self.batch_timestamp))

    def _send(self):
        """
        Send data to Influxdb. Data that can not be sent will be kept in queued.
        """
        # Check to see if we have a valid socket. If not, try to connect.
        try:
            if self.influx is None:
                self.log.debug("InfluxdbHandler: Socket is not connected. "
                               "Reconnecting.")
                self._connect()
            if self.influx is None:
                self.log.debug("InfluxdbHandler: Reconnect failed.")
            else:
                # build metrics data
                metrics = []
                for path in self.batch:
                    metrics.append({
                        "points": self.batch[path],
                        "name": path,
                        "columns": ["time", "value"]})
                # Send data to influxdb
                self.log.debug("InfluxdbHandler: writing %d series of data",
                               len(metrics))
                self.influx.write_points(metrics,
                                         time_precision=self.time_precision)

                # empty batch buffer
                self.batch = {}
                self.batch_count = 0
                self.time_multiplier = 1

        except Exception:
            self._close()
            if self.time_multiplier < 5:
                self.time_multiplier += 1
            self._throttle_error(
                "InfluxdbHandler: Error sending metrics, waiting for %ds.",
                2**self.time_multiplier)
            raise

    def _connect(self):
        """
        Connect to the influxdb server
        """

        try:
            # Open Connection
            self.influx = InfluxDBClient(self.hostname, self.port,
                                         self.username, self.password,
                                         self.database, self.ssl)
            # Log
            self.log.debug("InfluxdbHandler: Established connection to "
                           "%s:%d/%s.",
                           self.hostname, self.port, self.database)
        except Exception, ex:
            # Log Error
            self._throttle_error("InfluxdbHandler: Failed to connect to "
                                 "%s:%d/%s. %s",
                                 self.hostname, self.port, self.database, ex)
            # Close Socket
            self._close()
            return

    def _close(self):
        """
        Close the socket = do nothing for influx which is http stateless
        """
        self.influx = None



# coding=utf-8

"""
Collect metrics from Puppet Dashboard

#### Dependencies

 * urllib2

"""

import urllib2
import re
import diamond.collector


class PuppetDashboardCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(PuppetDashboardCollector,
                            self).get_default_config_help()
        config_help.update({
            'host': 'Hostname to collect from',
            'port': 'Port number to collect from',
            'path': 'Path to the dashboard',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(PuppetDashboardCollector, self).get_default_config()
        config.update({
            'host': 'localhost',
            'port': 5678,
            'path': 'puppetdashboard',
        })
        return config

    def collect(self):
        try:
            response = urllib2.urlopen("http://%s:%s/" % (
                self.config['host'], int(self.config['port'])))
        except Exception, e:
            self.log.error('Couldnt connect to puppet-dashboard: %s', e)
            return {}

        for line in response.readlines():
            line = line.strip()

            if line == "":
                continue

            try:
                regex = re.compile(
                    "<a href=\"/nodes/(?P<key>[\w.]+)\">(?P<count>[\d.]+)</a>")
                r = regex.search(line)
                results = r.groupdict()

                self.publish(results['key'], results['count'])
            except Exception, e:
                self.log.error('Couldnt parse the output: %s', e)



# coding=utf-8

"""
The XFSCollector collects XFS metrics using /proc/fs/xfs/stat.

#### Dependencies

 * /proc/fs/xfs/stat

"""

import diamond.collector
import sys


class XFSCollector(diamond.collector.Collector):

    PROC = '/proc/fs/xfs/stat'

    def get_default_config_help(self):
        config_help = super(XFSCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the xfs collector settings
        """
        config = super(XFSCollector, self).get_default_config()
        config.update({
            'path': 'xfs'
        })
        return config

    def collect(self):
        """
        Collect xfs stats.

        For an explanation of the following metrics visit
        http://xfs.org/index.php/Runtime_Stats
        https://github.com/torvalds/linux/blob/master/fs/xfs/xfs_stats.h
        """
        data_structure = {
            'extent_alloc': (
                'alloc_extent',
                'alloc_block',
                'free_extent',
                'free_block'
            ),
            'abt': (
                'lookup',
                'compare',
                'insrec',
                'delrec'
            ),
            'blk_map': (
                'read_ops',
                'write_ops',
                'unmap',
                'add_exlist',
                'del_exlist',
                'look_exlist',
                'cmp_exlist'
            ),
            'bmbt': (
                'lookup',
                'compare',
                'insrec',
                'delrec'
            ),
            'dir': (
                'lookup',
                'create',
                'remove',
                'getdents'
            ),
            'trans': (
                'sync',
                'async',
                'empty'
            ),
            'ig': (
                'ig_attempts',
                'ig_found',
                'ig_frecycle',
                'ig_missed',
                'ig_dup',
                'ig_reclaims',
                'ig_attrchg'
            ),
            'log': (
                'writes',
                'blocks',
                'noiclogs',
                'force',
                'force_sleep'
            ),
            'push_ail': (
                'try_logspace',
                'sleep_logspace',
                'pushes',
                'success',
                'pushbuf',
                'pinned',
                'locked',
                'flushing',
                'restarts',
                'flush'
            ),
            'xstrat': (
                'quick',
                'split'
            ),
            'rw': (
                'write_calls',
                'read_calls'
            ),
            'attr': (
                'get',
                'set',
                'remove',
                'list'
            ),
            'icluster': (
                'iflush_count',
                'icluster_flushcnt',
                'icluster_flushinode'
            ),
            'vnodes': (
                'vn_active',
                'vn_alloc',
                'vn_get',
                'vn_hold',
                'vn_rele',
                'vn_reclaim',
                'vn_remove',
                'vn_free'
            ),
            'buf': (
                'xb_get',
                'xb_create',
                'xb_get_locked',
                'xb_get_locked_waited',
                'xb_busy_locked',
                'xb_miss_locked',
                'xb_page_retries',
                'xb_page_found',
                'xb_get_read'
            ),
            'abtb2': (
                'xs_abtb_2_lookup',
                'xs_abtb_2_compare',
                'xs_abtb_2_insrec',
                'xs_abtb_2_delrec',
                'xs_abtb_2_newroot',
                'xs_abtb_2_killroot',
                'xs_abtb_2_increment',
                'xs_abtb_2_decrement',
                'xs_abtb_2_lshift',
                'xs_abtb_2_rshift',
                'xs_abtb_2_split',
                'xs_abtb_2_join',
                'xs_abtb_2_alloc',
                'xs_abtb_2_free',
                'xs_abtb_2_moves'
            ),
            'abtc2': (
                'xs_abtc_2_lookup',
                'xs_abtc_2_compare',
                'xs_abtc_2_insrec',
                'xs_abtc_2_delrec',
                'xs_abtc_2_newroot',
                'xs_abtc_2_killroot',
                'xs_abtc_2_increment',
                'xs_abtc_2_decrement',
                'xs_abtc_2_lshift',
                'xs_abtc_2_rshift',
                'xs_abtc_2_split',
                'xs_abtc_2_join',
                'xs_abtc_2_alloc',
                'xs_abtc_2_free',
                'xs_abtc_2_moves'
            ),
            'bmbt2': (
                'xs_bmbt_2_lookup',
                'xs_bmbt_2_compare',
                'xs_bmbt_2_insrec',
                'xs_bmbt_2_delrec',
                'xs_bmbt_2_newroot',
                'xs_bmbt_2_killroot',
                'xs_bmbt_2_increment',
                'xs_bmbt_2_decrement',
                'xs_bmbt_2_lshift',
                'xs_bmbt_2_rshift',
                'xs_bmbt_2_split',
                'xs_bmbt_2_join',
                'xs_bmbt_2_alloc',
                'xs_bmbt_2_free',
                'xs_bmbt_2_moves'
            ),
            'ibt2': (
                'lookup',
                'compare',
                'insrec',
                'delrec',
                'newroot',
                'killroot',
                'increment',
                'decrement',
                'lshift',
                'rshift',
                'split',
                'join',
                'alloc',
                'free',
                'moves'
            ),
            'fibt2': (
                'lookup',
                'compare',
                'insrec',
                'delrec',
                'newroot',
                'killroot',
                'increment',
                'decrement',
                'lshift',
                'rshift',
                'split',
                'join',
                'alloc',
                'free',
                'moves'
            ),
            'qm': (
                'xs_qm_dquot',
                'xs_qm_dquot_unused'
            ),

            'xpc': (
                'xs_xstrat_bytes',
                'xs_write_bytes',
                'xs_read_bytes'
            ),
            'debug': (
                'debug',
            )
        }

        f = open(self.PROC)
        new_stats = f.readlines()
        f.close()

        stats = {}
        for line in new_stats:
            items = line.rstrip().split()
            stats[items[0]] = [int(a) for a in items[1:]]

        for key in stats.keys():
            for item in enumerate(data_structure[key]):
                metric_name = '.'.join([key, item[1]])
                value = stats[key][item[0]]
                self.publish_counter(metric_name, value)



# coding=utf-8
"""
[Logentries: Log Management & Analytics Made Easy ](https://logentries.com/).
Send Diamond stats to your Logentries Account where you can monitor and alert
based on data in real time.
"""

from Handler import Handler
import logging
import urllib2
import json
from collections import deque


class LogentriesDiamondHandler(Handler):
    """
      Implements the abstract Handler class
    """

    def __init__(self, config=None):
        """
        New instance of LogentriesDiamondHandler class
        """

        Handler.__init__(self, config)
        self.log_token = self.config.get('log_token', None)
        self.queue_size = int(self.config['queue_size'])
        self.queue = deque([])
        if self.log_token is None:
            raise Exception

    def get_default_config_help(self):
        """
        Help text
        """
        config = super(LogentriesDiamondHandler,
                       self).get_default_config_help()

        config.update({
            'log_token':
                '[Your log token](https://logentries.com/doc/input-token/)',
            'queue_size': ''
        })

        return config

    def get_default_config(self):
        """
        Return default config for the handler
        """
        config = super(LogentriesDiamondHandler, self).get_default_config()

        config.update({
            'log_token': '',
            'queue_size': 100
        })

        return config

    def process(self, metric):
        """
        Process metric by sending it to datadog api
        """

        self.queue.append(metric)
        if len(self.queue) >= self.queue_size:
            logging.debug("Queue is full, sending logs to Logentries")
            self._send()

    def _send(self):
        """
        Convert message to a json object and send to Lognetries
        """
        while len(self.queue) > 0:
            metric = self.queue.popleft()
            topic, value, timestamp = str(metric).split()
            msg = json.dumps({"event": {topic: value}})
            req = urllib2.Request("https://js.logentries.com/v1/logs/" +
                                  self.log_token, msg)
            try:
                urllib2.urlopen(req)
            except urllib2.URLError, e:
                logging.error("Can't send log message to Logentries %s", e)



# coding=utf-8

"""
This is a meta handler to act as a shim for the new threading model. Please
do not try to use it as a normal handler
"""

from Handler import Handler
import Queue


class QueueHandler(Handler):

    def __init__(self, config=None, queue=None, log=None):
        # Initialize Handler
        Handler.__init__(self, config=config, log=log)

        self.queue = queue

    def __del__(self):
        """
        Ensure as many of the metrics as possible are sent to the handers on
        a shutdown
        """
        self._flush()

    def process(self, metric):
        return self._process(metric)

    def _process(self, metric):
        """
        We skip any locking code due to the fact that this is now a single
        process per collector
        """
        try:
            self.queue.put(metric, block=False)
        except Queue.Full:
            self._throttle_error('Queue full, check handlers for delays')

    def flush(self):
        return self._flush()

    def _flush(self):
        """
        We skip any locking code due to the fact that this is now a single
        process per collector
        """
        # Send a None down the queue to indicate a flush
        try:
            self.queue.put(None, block=False)
        except Queue.Full:
            self._throttle_error('Queue full, check handlers for delays')



# coding=utf-8

"""
Uses /proc/loadavg to collect data on load average

#### Dependencies

 * /proc/loadavg

"""

import diamond.collector
import re
import os
import multiprocessing
from diamond.collector import str_to_bool


class LoadAverageCollector(diamond.collector.Collector):

    PROC_LOADAVG = '/proc/loadavg'
    PROC_LOADAVG_RE = re.compile(r'([\d.]+) ([\d.]+) ([\d.]+) (\d+)/(\d+)')

    def get_default_config_help(self):
        config_help = super(LoadAverageCollector,
                            self).get_default_config_help()
        config_help.update({
            'simple':   'Only collect the 1 minute load average'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(LoadAverageCollector, self).get_default_config()
        config.update({
            'path':     'loadavg',
            'simple':   'False'
        })
        return config

    def collect(self):
        load01, load05, load15 = os.getloadavg()
        cpu_count = multiprocessing.cpu_count()

        if not str_to_bool(self.config['simple']):
            self.publish_gauge('01', load01, 2)
            self.publish_gauge('05', load05, 2)
            self.publish_gauge('15', load15, 2)
            self.publish_gauge('01_normalized', load01 / cpu_count, 2)
            self.publish_gauge('05_normalized', load05 / cpu_count, 2)
            self.publish_gauge('15_normalized', load15 / cpu_count, 2)
        else:
            self.publish_gauge('load', load01, 2)
            self.publish_gauge('load_normalized', load01 / cpu_count, 2)

        # Legacy: add process/thread counters provided by
        # /proc/loadavg (if available).
        if os.access(self.PROC_LOADAVG, os.R_OK):
            file = open(self.PROC_LOADAVG)
            for line in file:
                match = self.PROC_LOADAVG_RE.match(line)
                if match:
                    self.publish_gauge('processes_running',
                                       int(match.group(4)))
                    self.publish_gauge('processes_total', int(match.group(5)))
            file.close()



# coding=utf-8

"""
Send metrics to a [graphite](http://graphite.wikidot.com/) using the high
performace pickle interface.

Graphite is an enterprise-scale monitoring tool that runs well on cheap
hardware. It was originally designed and written by Chris Davis at Orbitz in
2006 as side project that ultimately grew to be a foundational monitoring tool.
In 2008, Orbitz allowed Graphite to be released under the open source Apache
2.0 license. Since then Chris has continued to work on Graphite and has
deployed it at other companies including Sears, where it serves as a pillar of
the e-commerce monitoring system. Today many
[large companies](http://graphite.readthedocs.org/en/latest/who-is-using.html)
use it.

"""

import struct

from graphite import GraphiteHandler

try:
    import cPickle as pickle
except ImportError:
    import pickle as pickle


class GraphitePickleHandler(GraphiteHandler):
    """
    Overrides the GraphiteHandler class
    Sending data to graphite using batched pickle format
    """

    def __init__(self, config=None):
        """
        Create a new instance of the GraphitePickleHandler
        """
        # Initialize GraphiteHandler
        GraphiteHandler.__init__(self, config)
        # Initialize Data
        self.batch = []
        # Initialize Options
        self.batch_size = int(self.config['batch'])

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(GraphitePickleHandler, self).get_default_config_help()

        config.update({
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(GraphitePickleHandler, self).get_default_config()

        config.update({
            'port': 2004,
        })

        return config

    def process(self, metric):
        # Convert metric to pickle format
        m = (metric.path, (metric.timestamp, metric.value))
        # Add the metric to the match
        self.batch.append(m)
        # If there are sufficient metrics, then pickle and send
        if len(self.batch) >= self.batch_size:
            # Log
            self.log.debug("GraphitePickleHandler: Sending batch size: %d",
                           self.batch_size)
            # Pickle the batch of metrics
            self.metrics = [self._pickle_batch()]
            # Send pickled batch
            self._send()
            # Flush the metric pack down the wire
            self.flush()
            # Clear Batch
            self.batch = []

    def _pickle_batch(self):
        """
        Pickle the metrics into a form that can be understood
        by the graphite pickle connector.
        """
        # Pickle
        payload = pickle.dumps(self.batch)

        # Pack Message
        header = struct.pack("!L", len(payload))
        message = header + payload

        # Return Message
        return message



# coding=utf-8

"""
Emulate a gmetric client for usage with
[Ganglia Monitoring System](http://ganglia.sourceforge.net/)
"""

from Handler import Handler
import logging
try:
    import gmetric
except ImportError:
    gmetric = None


class GmetricHandler(Handler):
    """
    Implements the abstract Handler class, sending data the same way that
    gmetric does.
    """

    def __init__(self, config=None):
        """
        Create a new instance of the GmetricHandler class
        """
        # Initialize Handler
        Handler.__init__(self, config)

        if gmetric is None:
            logging.error("Failed to load gmetric module")
            return

        # Initialize Data
        self.socket = None

        # Initialize Options
        self.host = self.config['host']
        self.port = int(self.config['port'])
        self.protocol = self.config['protocol']
        if not self.protocol:
            self.protocol = 'udp'

        # Initialize
        self.gmetric = gmetric.Gmetric(self.host, self.port, self.protocol)

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(GmetricHandler, self).get_default_config_help()

        config.update({
            'host': 'Hostname',
            'port': 'Port',
            'protocol': 'udp or tcp',
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(GmetricHandler, self).get_default_config()

        config.update({
            'host': 'localhost',
            'port': 8651,
            'protocol': 'udp',
        })

        return config

    def __del__(self):
        """
        Destroy instance of the GmetricHandler class
        """
        self._close()

    def process(self, metric):
        """
        Process a metric by sending it to a gmond instance
        """
        # Just send the data as a string
        self._send(metric)

    def _send(self, metric):
        """
        Send data to gmond.
        """
        metric_name = self.get_name_from_path(metric.path)
        tmax = "60"
        dmax = "0"
        slope = "both"
        # FIXME: Badness, shouldn't *assume* double type
        metric_type = "double"
        units = ""
        group = ""
        self.gmetric.send(metric_name,
                          metric.value,
                          metric_type,
                          units,
                          slope,
                          tmax,
                          dmax,
                          group)

    def _close(self):
        """
        Close the connection
        """
        self.gmetric = None



# coding=utf-8

"""

Collects metrics from a mesos instance. By default,
the collector is set up to query the mesos-master via
port 5050. Set the port to 5051 for mesos-slaves.

#### Example Configuration

```
    host = localhost
    port = 5050
```
"""

import diamond.collector
import json
import urllib2


class MesosCollector(diamond.collector.Collector):

    METRICS_PATH = "metrics/snapshot"

    def get_default_config_help(self):
        config_help = super(MesosCollector,
                            self).get_default_config_help()
        config_help.update({
            'host': 'Hostname',
            'port': 'Port (default is 5050; please set to 5051 for mesos-slave)'
        })
        return config_help

    def get_default_config(self):
        config = super(MesosCollector, self).get_default_config()
        config.update({
            'host': 'localhost',
            'port': 5050,
            'path': 'mesos'
        })
        return config

    def __init__(self, *args, **kwargs):
        super(MesosCollector, self).__init__(*args, **kwargs)

    def collect(self):
        metrics = self.get_metrics()

        for k, v in metrics.iteritems():
            key = self.clean_up(k)
            self.publish(key, v)

    def get_metrics(self):
        try:
            url = "http://%s:%s/%s" % (self.config['host'],
                                       self.config['port'],
                                       self.METRICS_PATH)

            return json.load(urllib2.urlopen(url))
        except (urllib2.HTTPError, ValueError), err:
            self.log.error('Unable to read JSON response: %s' % err)
            return {}

    def clean_up(self, text):
        return text.replace('/', '.')



# coding=utf-8

"""
The MemoryDockerCollector collects memory statistics from docker containers

#### Dependencies

 * docker

"""

import os
import sys

try:
    import docker
except ImportError:
    docker = None

sys.path.append(os.path.join(os.path.dirname(os.path.dirname(__file__)),
                             'memory_cgroup'))
from memory_cgroup import MemoryCgroupCollector


class MemoryDockerCollector(MemoryCgroupCollector):

    def collect(self):
        if docker is None:
            self.log.error('Unable to import docker')
            return

        self.containers = dict(
            (c['Id'], c['Names'][0][1:])
            for c in docker.Client().containers(all=True)
            if c['Names'] is not None)
        return super(MemoryDockerCollector, self).collect()

    def publish(self, metric_name, value, metric_type):
        for container_id, container_name in self.containers.items():
            metric_name = metric_name.replace(
                'docker.' + container_id + '.',
                'docker.' + container_name + '.')
        return super(MemoryDockerCollector, self).publish(
            metric_name, value, metric_type)



# coding=utf-8

"""
The SNMPRawCollector is designed for collecting data from SNMP-enables devices,
using a set of specified OIDs

#### Configuration

Below is an example configuration for the SNMPRawCollector. The collector
can collect data any number of devices by adding configuration sections
under the *devices* header. By default the collector will collect every 60
seconds. This might be a bit excessive and put unnecessary load on the
devices being polled. You may wish to change this to every 300 seconds. However
you need modify your graphite data retentions to handle this properly.

```
    # Options for SNMPRawCollector
    enabled = True
    interval = 60

    [devices]

    # Start the device configuration
    # Note: this name will be used in the metric path.
    [[my-identification-for-this-host]]
    host = localhost
    port = 161
    community = public

    # Start the OID list for this device
    # Note: the value part will be used in the metric path.
    [[[oids]]]
    1.3.6.1.4.1.2021.10.1.3.1 = cpu.load.1min
    1.3.6.1.4.1.2021.10.1.3.2 = cpu.load.5min
    1.3.6.1.4.1.2021.10.1.3.3 = cpu.load.15min

    # If you want another host, you can. But you probably won't need it.
    [[another-identification]]
    host = router1.example.com
    port = 161
    community = public
    [[[oids]]]
    oid = metric.path
    oid = metric.path
```

Note: If you modify the SNMPRawCollector configuration, you will need to
restart diamond.

#### Dependencies

 * pysmnp (which depends on pyasn1 0.1.7 and pycrypto)

"""

import os
import sys
import time

sys.path.insert(0, os.path.join(os.path.dirname(os.path.dirname(__file__)),
                                'snmp'))
from snmp import SNMPCollector as parent_SNMPCollector
from diamond.metric import Metric


class SNMPRawCollector(parent_SNMPCollector):

    def process_config(self):
        super(SNMPRawCollector, self).process_config()
        # list to save non-existing oid's per device, to avoid repetition of
        # errors in logging. Signal HUP to diamond/collector to flush this
        self.skip_list = []

    def get_default_config(self):
        """
        Override SNMPCollector.get_default_config method to provide
        default_config for the SNMPInterfaceCollector
        """
        default_config = super(SNMPRawCollector,
                               self).get_default_config()
        default_config.update({
            'oids': {},
            'path_prefix': 'servers',
            'path_suffix': 'snmp',
        })
        return default_config

    def _precision(self, value):
        """
        Return the precision of the number
        """
        value = str(value)
        decimal = value.rfind('.')
        if decimal == -1:
            return 0
        return len(value) - decimal - 1

    def _skip(self, device, oid, reason=None):
        self.skip_list.append((device, oid))
        if reason is not None:
            self.log.warn('Muted \'{0}\' on \'{1}\', because: {2}'.format(
                oid, device, reason))

    def _get_value_walk(self, device, oid, host, port, community):
        data = self.walk(oid, host, port, community)

        if data is None:
            self._skip(device, oid, 'device down (#2)')
            return

        self.log.debug('Data received from WALK \'{0}\': [{1}]'.format(
            device, data))

        if len(data) != 1:
            self._skip(
                device,
                oid,
                'unexpected response, data has {0} entries'.format(
                    len(data)))
            return

        # because we only allow 1-key dicts, we can pick with absolute index
        value = data.items()[0][1]
        return value

    def _get_value(self, device, oid, host, port, community):
        data = self.get(oid, host, port, community)

        if data is None:
            self._skip(device, oid, 'device down (#1)')
            return

        self.log.debug('Data received from GET \'{0}\': [{1}]'.format(
            device, data))

        if len(data) == 0:
            self._skip(device, oid, 'empty response, device down?')
            return

        if oid not in data:
            # oid is not even in hierarchy, happens when using 9.9.9.9
            # but not when using 1.9.9.9
            self._skip(device, oid, 'no object at OID (#1)')
            return

        value = data[oid]
        if value == 'No Such Object currently exists at this OID':
            self._skip(device, oid, 'no object at OID (#2)')
            return

        if value == 'No Such Instance currently exists at this OID':
            return self._get_value_walk(device, oid, host, port, community)

        return value

    def collect_snmp(self, device, host, port, community):
        """
        Collect SNMP interface data from device
        """
        self.log.debug(
            'Collecting raw SNMP statistics from device \'{0}\''.format(device))

        dev_config = self.config['devices'][device]
        if 'oids' in dev_config:
            for oid, metricName in dev_config['oids'].items():

                if (device, oid) in self.skip_list:
                    self.log.debug(
                        'Skipping OID \'{0}\' ({1}) on device \'{2}\''.format(
                            oid, metricName, device))
                    continue

                timestamp = time.time()
                value = self._get_value(device, oid, host, port, community)
                if value is None:
                    continue

                self.log.debug(
                    '\'{0}\' ({1}) on device \'{2}\' - value=[{3}]'.format(
                        oid, metricName, device, value))

                path = '.'.join([self.config['path_prefix'], device,
                                 self.config['path_suffix'], metricName])
                metric = Metric(path=path, value=value, timestamp=timestamp,
                                precision=self._precision(value),
                                metric_type='GAUGE')
                self.publish_metric(metric)



# coding=utf-8

"""
Processes OpenVPN metrics. This collector can process multiple OpenVPN
instances (even from a server box). In addition to the path, you may
also specify a name of the instance.

You can use both the status file or the tcp management connection to
retrieve the metrics.

To parse the status file::

    instances = file:///var/log/openvpn/status.log

Or, to override the name (now "status"):

    instances = file:///var/log/openvpn/status.log?developers

To use the management connection::

    instances = tcp://127.0.0.1:1195

Or, to override the name (now "127_0_0_1"):

    instances = tcp://127.0.0.1:1195?developers

You can also specify multiple and mixed instances::

    instances = file:///var/log/openvpn/openvpn.log, tcp://10.0.0.1:1195?admins

#### Dependencies

 * urlparse

"""

import socket
import diamond.collector
import os.path
import urlparse
import time


class OpenVPNCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(OpenVPNCollector, self).get_default_config_help()
        config_help.update({
            'instances': 'List of instances to collect stats from',
            'timeout': 'network timeout'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(OpenVPNCollector, self).get_default_config()
        config.update({
            'path':      'openvpn',
            'instances': 'file:///var/log/openvpn/status.log',
            'timeout':   '10',
        })
        return config

    def parse_url(self, uri):
        """
        Convert urlparse from a python 2.4 layout to a python 2.7 layout
        """
        parsed = urlparse.urlparse(uri)
        if 'scheme' not in parsed:
            class Object(object):
                pass
            newparsed = Object()
            newparsed.scheme = parsed[0]
            newparsed.netloc = parsed[1]
            newparsed.path = parsed[2]
            newparsed.params = parsed[3]
            newparsed.query = parsed[4]
            newparsed.fragment = parsed[5]
            newparsed.username = ''
            newparsed.password = ''
            newparsed.hostname = ''
            newparsed.port = ''
            parsed = newparsed
        return parsed

    def collect(self):
        if isinstance(self.config['instances'], basestring):
            instances = [self.config['instances']]
        else:
            instances = self.config['instances']

        for uri in instances:
            parsed = self.parse_url(uri)
            collect = getattr(self, 'collect_%s' % (parsed.scheme,), None)
            if collect:
                collect(uri)
            else:
                self.log.error('OpenVPN no handler for %s', uri)

    def collect_file(self, uri):
        parsed = self.parse_url(uri)
        filename = parsed.path
        if '?' in filename:
            filename, name = filename.split('?')
        else:
            name = os.path.splitext(os.path.basename(filename))[0]

        if not os.access(filename, os.R_OK):
            self.log.error('OpenVPN collect failed: unable to read "%s"',
                           filename)
            return
        else:
            self.log.info('OpenVPN parsing "%s" file: %s', name, filename)

        fd = open(filename, 'r')
        lines = fd.readlines()
        fd.close()

        self.parse(name, lines)

    def collect_tcp(self, uri):
        parsed = self.parse_url(uri)
        try:
            host, port = parsed.netloc.split(':')
            port = int(port)
        except ValueError:
            self.log.error('OpenVPN expected host:port in URI, got "%s"',
                           parsed.netloc)
            return

        if '?' in parsed.path:
            name = parsed.path[1:]
        else:
            name = host.replace('.', '_')

        self.log.info('OpenVPN parsing "%s" tcp: %s:%d', name, host, port)

        try:
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            server.settimeout(int(self.config['timeout']))
            server.connect((host, port))

            fd = server.makefile('rb')
            line = fd.readline()
            if not line.startswith('>INFO:OpenVPN'):
                self.log.debug('OpenVPN received: %s', line.rstrip())
                self.log.error('OpenVPN protocol error')
                server.close()
                return

            server.send('status\r\n')

            lines = []
            while True:
                line = fd.readline()
                lines.append(line)
                if line.strip() == 'END':
                    break

            # Hand over data to the parser
            self.parse(name, lines)

            # Bye
            server.close()

        except socket.error, e:
            self.log.error('OpenVPN management connection error: %s', e)
            return

    def parse(self, name, lines):
        for line in lines:
            self.log.debug('OpenVPN: %s', line.rstrip())

        time.sleep(0.5)

        number_connected_clients = 0
        section = ''
        heading = []
        for line in lines:
            if line.strip() == 'END':
                break
            elif line.lower().startswith('openvpn statistics'):
                section = 'statistics'
            elif line.lower().startswith('openvpn client list'):
                section = 'clients'
            elif line.lower().startswith('routing table'):
                # ignored
                section = ''
            elif line.lower().startswith('global stats'):
                section = 'global'
            elif ',' in line:
                key, value = line.split(',', 1)
                if key.lower() == 'updated':
                    continue

                if section == 'statistics':
                    # All values here are numeric
                    self.publish_number('.'.join([
                        name,
                        'global',
                        key, ]), value)

                elif section == 'clients':
                    # Clients come with a heading
                    if not heading:
                        heading = line.strip().split(',')
                    else:
                        info = {}
                        number_connected_clients += 1
                        for k, v in zip(heading, line.strip().split(',')):
                            info[k.lower()] = v

                        self.publish_number('.'.join([
                            name,
                            section,
                            info['common name'].replace('.', '_'),
                            'bytes_rx']), info['bytes received'])
                        self.publish_number('.'.join([
                            name,
                            section,
                            info['common name'].replace('.', '_'),
                            'bytes_tx']), info['bytes sent'])

                elif section == 'global':
                    # All values here are numeric
                    self.publish_number('.'.join([
                        name,
                        section,
                        key, ]), value)

            elif line.startswith('END'):
                break
        self.publish('%s.clients.connected' % name, number_connected_clients)

    def publish_number(self, key, value):
        key = key.replace('/', '-').replace(' ', '_').lower()
        try:
            value = long(value)
        except ValueError:
            self.log.error('OpenVPN expected a number for "%s", got "%s"',
                           key, value)
            return
        else:
            self.publish(key, value)



# coding=utf-8

"""
Collect the monit stats and report on cpu/memory for monitored processes

#### Dependencies

 * monit serving up /_status

"""

import urllib2
import base64

from xml.dom.minidom import parseString

import diamond.collector
from diamond.collector import str_to_bool


class MonitCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(MonitCollector, self).get_default_config_help()
        config_help.update({
            'send_totals': 'Send cpu and memory totals',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(MonitCollector, self).get_default_config()
        config.update({
            'host':         '127.0.0.1',
            'port':         2812,
            'user':         'monit',
            'passwd':       'monit',
            'path':         'monit',
            'byte_unit':    ['byte'],
            'send_totals':  False,
        })
        return config

    def collect(self):
        url = 'http://%s:%i/_status?format=xml' % (self.config['host'],
                                                   int(self.config['port']))
        try:
            request = urllib2.Request(url)

            #
            # shouldn't need to check this
            base64string = base64.encodestring('%s:%s' % (
                self.config['user'], self.config['passwd'])).replace('\n', '')
            request.add_header("Authorization", "Basic %s" % base64string)
            response = urllib2.urlopen(request)
        except urllib2.HTTPError, err:
            self.log.error("%s: %s", err, url)
            return

        metrics = {}

        try:
            dom = parseString("".join(response.readlines()))
        except:
            self.log.error("Got an empty response from the monit server")
            return

        for svc in dom.getElementsByTagName('service'):
            if int(svc.getAttribute('type')) == 3:
                name = svc.getElementsByTagName('name')[0].firstChild.data
                status = svc.getElementsByTagName('status')[0].firstChild.data
                monitor = svc.getElementsByTagName(
                    'monitor')[0].firstChild.data
                if status == '0' and monitor == '1':
                    try:
                        uptime = svc.getElementsByTagName(
                            'uptime')[0].firstChild.data
                        metrics["%s.uptime" % name] = uptime

                        cpu = svc.getElementsByTagName(
                            'cpu')[0].getElementsByTagName(
                            'percent')[0].firstChild.data
                        metrics["%s.cpu.percent" % name] = cpu
                        if str_to_bool(self.config['send_totals']):
                            cpu_total = svc.getElementsByTagName(
                                'cpu')[0].getElementsByTagName(
                                'percenttotal')[0].firstChild.data
                            metrics["%s.cpu.percent_total" % name] = cpu_total

                        mem = int(svc.getElementsByTagName(
                            'memory')[0].getElementsByTagName(
                            'kilobyte')[0].firstChild.data)
                        for unit in self.config['byte_unit']:
                            metrics["%s.memory.%s_usage" % (name, unit)] = (
                                diamond.convertor.binary.convert(
                                    value=mem,
                                    oldUnit='kilobyte',
                                    newUnit=unit))
                        metrics["%s.uptime" % name] = uptime
                        if str_to_bool(self.config['send_totals']):
                            mem_total = int(svc.getElementsByTagName(
                                'memory')[0].getElementsByTagName(
                                'kilobytetotal')[0].firstChild.data)
                            for unit in self.config['byte_unit']:
                                metrics["%s.memory_total.%s_usage" % (
                                    name, unit)] = (
                                    diamond.convertor.binary.convert(
                                        value=mem_total,
                                        oldUnit='kilobyte',
                                        newUnit=unit))

                    except:
                        pass

        for key in metrics:
            self.publish(key, metrics[key])



# coding=utf-8
"""
Output the collected values to AWS CloudWatch

Automatically adds the InstanceId Dimension

#### Dependencies

 * [boto](http://boto.readthedocs.org/en/latest/index.html)

#### Configuration

Enable this handler

 * handers = diamond.handler.cloudwatch.cloudwatchHandler

Example Config:

[[cloudwatchHandler]]
region = us-east-1

[[[LoadAvg01]]]
collector = loadavg
metric = 01
namespace = MachineLoad
name = Avg01
unit = None

[[[LoadAvg05]]]
collector = loadavg
metric = 05
namespace = MachineLoad
name = Avg05
unit = None
"""

import sys
import datetime

from Handler import Handler
from configobj import Section

try:
    import boto
    import boto.ec2.cloudwatch
    import boto.utils
except ImportError:
    boto = None


class cloudwatchHandler(Handler):
    """
      Implements the abstract Handler class
      Sending data to a AWS CloudWatch
    """

    def __init__(self, config=None):
        """
          Create a new instance of cloudwatchHandler class
        """

        # Initialize Handler
        Handler.__init__(self, config)

        if not boto:
            self.log.error(
                "CloudWatch: Boto is not installed, please install boto.")
            return

        # Initialize Data
        self.connection = None

        # Initialize Options
        self.region = self.config['region']
        instances = boto.utils.get_instance_metadata()
        if 'instance-id' not in instances:
            self.log.error('CloudWatch: Failed to load instance metadata')
            return
        self.instance_id = instances['instance-id']
        self.log.debug("Setting InstanceId: " + self.instance_id)

        self.valid_config = ('region', 'collector', 'metric', 'namespace',
                             'name', 'unit')

        self.rules = []
        for key_name, section in self.config.items():
            if section.__class__ is Section:
                keys = section.keys()
                rules = {}
                for key in keys:
                    if key not in self.valid_config:
                        self.log.warning("invalid key %s in section %s",
                                         key, section.name)
                    else:
                        rules[key] = section[key]

                self.rules.append(rules)

        # Create CloudWatch Connection
        self._bind()

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(cloudwatchHandler, self).get_default_config_help()

        config.update({
            'region': '',
            'metric': '',
            'namespace': '',
            'name': '',
            'unit': '',
            'collector': '',
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(cloudwatchHandler, self).get_default_config()

        config.update({
            'region': 'us-east-1',
            'collector': 'loadavg',
            'metric': '01',
            'namespace': 'MachineLoad',
            'name': 'Avg01',
            'unit': 'None',
        })

        return config

    def _bind(self):
        """
           Create CloudWatch Connection
        """

        self.log.debug(
            "CloudWatch: Attempting to connect to CloudWatch at Region: %s",
            self.region)
        try:
            self.connection = boto.ec2.cloudwatch.connect_to_region(
                self.region)
            self.log.debug(
                "CloudWatch: Succesfully Connected to CloudWatch at Region: %s",
                self.region)
        except boto.exception.EC2ResponseError:
            self.log.error('CloudWatch: CloudWatch Exception Handler: ')

    def __del__(self):
        """
          Destroy instance of the cloudWatchHandler class
        """
        try:
            self.connection = None
        except AttributeError:
            pass

    def process(self, metric):
        """
          Process a metric and send it to CloudWatch
        """
        if not boto:
            return

        collector = str(metric.getCollectorPath())
        metricname = str(metric.getMetricPath())
        timestamp = datetime.datetime.fromtimestamp(metric.timestamp)

        # Send the data as ......

        for rule in self.rules:
            self.log.debug(
                "Comparing Collector: [%s] with (%s) "
                "and Metric: [%s] with (%s)",
                str(rule['collector']),
                collector,
                str(rule['metric']),
                metricname
            )

            if ((str(rule['collector']) == collector and
                 str(rule['metric']) == metricname)):
                self.log.debug(
                    "CloudWatch: Attempting to publish metric: %s to %s "
                    "with value (%s) @%s",
                    rule['name'],
                    rule['namespace'],
                    str(metric.value),
                    str(metric.timestamp)
                )
                try:
                    self.connection.put_metric_data(
                        str(rule['namespace']),
                        str(rule['name']),
                        str(metric.value),
                        timestamp, str(rule['unit']),
                        {'InstanceId': self.instance_id})
                    self.log.debug(
                        "CloudWatch: Successfully published metric: %s to"
                        " %s with value (%s)",
                        rule['name'],
                        rule['namespace'],
                        str(metric.value)
                    )
                except AttributeError, e:
                    self.log.error(
                        "CloudWatch: Failed publishing - %s ", str(e))
                except Exception:  # Rough connection re-try logic.
                    self.log.error(
                        "CloudWatch: Failed publishing - %s ",
                        str(sys.exc_info()[0]))
                    self._bind()



# coding=utf-8

import configobj
import os


def str_to_bool(value):
    """
    Converts string truthy/falsey strings to a bool
    Empty strings are false
    """
    if isinstance(value, basestring):
        value = value.strip().lower()
        if value in ['true', 't', 'yes', 'y']:
            return True
        elif value in ['false', 'f', 'no', 'n', '']:
            return False
        else:
            raise NotImplementedError("Unknown bool %s" % value)

    return value


def load_config(configfile):
    """
    Load the full config / merge splitted configs if configured
    """

    configfile = os.path.abspath(configfile)
    config = configobj.ConfigObj(configfile)

    config_extension = '.conf'

    #########################################################################
    # Load up other config files
    #########################################################################

    if 'configs' in config:
        config_extension = config['configs'].get('extension', config_extension)

        # Load other configs
        if 'path' in config['configs']:
            for cfgfile in os.listdir(config['configs']['path']):
                cfgfile = os.path.join(config['configs']['path'],
                                       cfgfile)
                cfgfile = os.path.abspath(cfgfile)
                if not cfgfile.endswith(config_extension):
                    continue
                newconfig = configobj.ConfigObj(cfgfile)
                config.merge(newconfig)

    #########################################################################

    if 'server' not in config:
        raise Exception('Failed to load config file %s!' % configfile)

    #########################################################################
    # Load up handler specific configs
    #########################################################################

    if 'handlers' not in config:
        config['handlers'] = configobj.ConfigObj()

    if 'handlers_config_path' in config['server']:
        handlers_config_path = config['server']['handlers_config_path']
        if os.path.exists(handlers_config_path):
            for cfgfile in os.listdir(handlers_config_path):
                cfgfile = os.path.join(handlers_config_path, cfgfile)
                cfgfile = os.path.abspath(cfgfile)
                if not cfgfile.endswith(config_extension):
                    continue
                filename = os.path.basename(cfgfile)
                handler = os.path.splitext(filename)[0]

                if handler not in config['handlers']:
                    config['handlers'][handler] = configobj.ConfigObj()

                newconfig = configobj.ConfigObj(cfgfile)
                config['handlers'][handler].merge(newconfig)

    #########################################################################
    # Load up Collector specific configs
    #########################################################################

    if 'collectors' not in config:
        config['collectors'] = configobj.ConfigObj()

    if 'collectors_config_path' in config['server']:
        collectors_config_path = config['server']['collectors_config_path']
        if os.path.exists(collectors_config_path):
            for cfgfile in os.listdir(collectors_config_path):
                cfgfile = os.path.join(collectors_config_path, cfgfile)
                cfgfile = os.path.abspath(cfgfile)
                if not cfgfile.endswith(config_extension):
                    continue
                filename = os.path.basename(cfgfile)
                collector = os.path.splitext(filename)[0]

                if collector not in config['collectors']:
                    config['collectors'][collector] = configobj.ConfigObj()

                try:
                    newconfig = configobj.ConfigObj(cfgfile)
                except Exception, e:
                    raise Exception("Failed to load config file %s due to %s" %
                                    (cfgfile, e))

                config['collectors'][collector].merge(newconfig)

    # Convert enabled to a bool
    for collector in config['collectors']:
        if 'enabled' in config['collectors'][collector]:
            config['collectors'][collector]['enabled'] = str_to_bool(
                config['collectors'][collector]['enabled']
            )

    #########################################################################

    return config



# coding=utf-8

"""
Uses /sys/fs/vmsfs to collect host-global data on VMS memory usage

#### Dependencies

 * /sys/fs/vmsfs <- vmsfs package, vmsfs mounted

"""

import diamond.collector
import os


class VMSFSCollector(diamond.collector.Collector):

    SYSFS = '/sys/fs/vmsfs'

    VMSFS_STATS = {
        'resident': ('cur_resident', 4096),
        'allocated': ('cur_allocated', 4096)
    }

    def vmsfs_stats_read(self, filename):
        stats = {}

        # Open vmsfs sys info.
        stats_fd = None
        try:
            stats_fd = open(filename)

            for line in stats_fd:
                tokens = line.split()
                stats[tokens[0][0:-1]] = long(tokens[1])
        except:
            if stats_fd:
                stats_fd.close()

        return stats

    def vmsfs_stats_dispatch(self, filename, prefix=''):
        stats = self.vmsfs_stats_read(filename)
        for stat in self.VMSFS_STATS:
            name = self.VMSFS_STATS[stat][0]
            scale = self.VMSFS_STATS[stat][1]
            if name in stats:
                self.publish(prefix + name, stats[name] * scale)

    def get_default_config_help(self):
        config_help = super(VMSFSCollector, self).get_default_config_help()
        config_help.update({
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(VMSFSCollector, self).get_default_config()
        config.update({
            'path':     'vmsfs'
        })
        return config

    def collect(self):
        if not os.access(self.SYSFS, os.R_OK | os.X_OK):
            return None

        # Dispatch total stats.
        self.vmsfs_stats_dispatch(os.path.join(self.SYSFS, 'stats'))

        # Dispatch per-generation stats.
        # NOTE: We do not currently report the per-generation statistics to
        # diamond. This is because we do not have a good strategy for
        # aggregating generation data and exposing it in a sensible way. There
        # are three strategies:
        #  1) Collect everything at the host level.
        #     The problem here is that the number of metrics will explode for
        #     that individual host (and keep growing).
        #  2) Collect at the top-level (one virtual host per generation).
        #     Then the problem is finding the generation through UI tools, etc.
        #  3) Figure out some way to put the stats in each instance associated
        #     with that generation.
        # We favor (2) currently, but there's not much value in implementing it
        # until it can be exposed to the user.
        if False:
            TO_IGNORE = ('stats', 'version',
                         '00000000-0000-0000-0000-000000000000')
            files = os.listdir(self.SYSFS)
            for f in files:
                if f not in TO_IGNORE:
                    self.vmsfs_stats_dispatch('/sys/fs/vmsfs/' + f,
                                              prefix=('%s.' % f))



# coding=utf-8

"""
Openstack swift collector.

#### Dependencies

 * swift-dispersion-report commandline tool (for dispersion report)
   if using this, make sure swift.conf and dispersion.conf are readable by
   diamond also get an idea of the runtime of a swift-dispersion-report call
   and make sure the collect interval is high enough to avoid contention.
 * swift commandline tool (for container_metrics)

both of these should come installed with swift
"""

import diamond.collector
from subprocess import Popen, PIPE

try:
    import json
except ImportError:
    import simplejson as json


class OpenstackSwiftCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(OpenstackSwiftCollector,
                            self).get_default_config_help()
        config_help.update({
            'enable_dispersion_report': 'gather swift-dispersion-report ' +
                                        'metrics (default False)',
            'enable_container_metrics': 'gather containers metrics ' +
                                        '(# objects, bytes used, ' +
                                        'x_timestamp. default True)',
            'auth_url': 'authentication url (for enable_container_metrics)',
            'account': 'swift auth account (for enable_container_metrics)',
            'user': 'swift auth user (for enable_container_metrics)',
            'password': 'swift auth password (for enable_container_metrics)',
            'containers': 'containers on which to count number of objects, ' +
                          'space separated list (for enable_container_metrics)'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(OpenstackSwiftCollector, self).get_default_config()
        config.update({
            'path': 'openstackswift',
            'enable_dispersion_report': False,
            'enable_container_metrics': True,
            # don't use the threaded model with this one.
            # for some reason it crashes.
            'interval': 1200,  # by default, every 20 minutes
        })
        return config

    def collect(self):
        # dispersion report.  this can take easily >60s. beware!
        if (self.config['enable_dispersion_report']):
            p = Popen(
                ['swift-dispersion-report', '-j'],
                stdout=PIPE,
                stderr=PIPE)
            stdout, stderr = p.communicate()
            self.publish('dispersion.errors', len(stderr.split('\n')) - 1)
            data = json.loads(stdout)
            for t in ('object', 'container'):
                for (k, v) in data[t].items():
                    self.publish('dispersion.%s.%s' % (t, k), v)

        # container metrics returned by stat <container>
        if(self.config['enable_container_metrics']):
            account = '%s:%s' % (self.config['account'], self.config['user'])
            for container in self.config['containers'].split(','):
                cmd = ['swift', '-A', self.config['auth_url'],
                       '-U', account,
                       '-K', self.config['password'],
                       'stat', container]
                p = Popen(cmd, stdout=PIPE, stderr=PIPE)
                stdout, stderr = p.communicate()
                stats = {}
                # stdout is some lines in 'key   : val' format
                for line in stdout.split('\n'):
                    if line:
                        line = line.split(':', 2)
                        stats[line[0].strip()] = line[1].strip()
                key = 'container_metrics.%s.%s' % (self.config['account'],
                                                   container)
                self.publish('%s.objects' % key, stats['Objects'])
                self.publish('%s.bytes' % key, stats['Bytes'])
                self.publish('%s.x_timestamp' % key, stats['X-Timestamp'])



# coding=utf-8

"""
The DockerCollector uses the docker stats api to
collect data about docker and the containers.

#### Dependencies

* docker -- Install via `pip install docker-py`
  Source https://github.com/docker/docker-py
"""

import json
import diamond.collector

try:
    import docker
except ImportError:
    docker = None


class DockerCollector(diamond.collector.Collector):

    METRICS = {
        # memory stats
        "memory_stats.stats.total_rss": "RSS_byte",
        "memory_stats.stats.total_cache": "cache_byte",
        "memory_stats.stats.total_swap": "swap_byte",
        "memory_stats.stats.total_pgpgin": "pagein_count",
        "memory_stats.stats.total_pgpgout": "pageout_count",

        # cpu stats
        "cpu_stats.cpu_usage.total_usage": "cpu.total",
        "cpu_stats.cpu_usage.usage_in_kernelmode": "cpu.kernelmode",
        "cpu_stats.cpu_usage.usage_in_usermode": "cpu.usermode",
        "cpu_stats.system_cpu_usage": "cpu.system",
    }

    def get_default_config_help(self):
        return super(DockerCollector, self).get_default_config_help()

    def get_default_config(self):
        config = super(DockerCollector, self).get_default_config()
        config.update({
            'path': 'docker'
        })
        return config

    def get_value(self, path, dictionary):
        keys = path.split(".")
        cur = dictionary
        for key in keys:
            if not isinstance(cur, dict):
                raise Exception("metric '{0}' does not exist".format(path))
            cur = cur.get(key)
            if cur is None:
                break
        return cur

    def collect(self):
        if docker is None:
            self.log.error('Unable to import docker')

        # Collect info
        results = {}
        client = docker.Client(version='auto')

        # Top level stats
        running_containers = client.containers()
        results['containers_running_count'] = (
            len(running_containers), 'GAUGE')

        all_containers = client.containers(all=True)
        results['containers_stopped_count'] = (
            len(all_containers) - len(running_containers), 'GAUGE')

        images_count = len(set(client.images(quiet=True)))
        results['images_count'] = (images_count, 'GAUGE')

        dangling_images_count = len(set(client.images(
            quiet=True, all=True, filters={'dangling': True})))
        results['images_dangling_count'] = (dangling_images_count, 'GAUGE')

        # Collect memory and cpu stats
        for container in running_containers:
            name = "containers." + "".join(container['Names'][0][1:])
            s = client.stats(container["Id"])
            stat = json.loads(s.next())
            for path in self.METRICS:
                val = self.get_value(path, stat)
                if val is not None:
                    metric_key = ".".join([name, self.METRICS.get(path)])
                    results[metric_key] = (val, 'GAUGE')
            s.close()

        for name in sorted(results.keys()):
            (value, metric_type) = results[name]
            self.publish(name, value, metric_type=metric_type)



# coding=utf-8

"""
Write the collected stats to a locally stored log file. Rotate the log file
every night and remove after 7 days.
"""

from Handler import Handler
import logging
import logging.handlers


class ArchiveHandler(Handler):
    """
    Implements the Handler abstract class, archiving data to a log file
    """

    def __init__(self, config):
        """
        Create a new instance of the ArchiveHandler class
        """
        # Initialize Handler
        Handler.__init__(self, config)

        # Create Archive Logger
        self.archive = logging.getLogger('archive')
        self.archive.setLevel(logging.DEBUG)
        self.archive.propagate = self.config['propagate']
        # Create Archive Log Formatter
        formatter = logging.Formatter('%(message)s')
        # Create Archive Log Handler
        handler = logging.handlers.TimedRotatingFileHandler(
            filename=self.config['log_file'],
            when='midnight',
            interval=1,
            backupCount=int(self.config['days']),
            encoding=self.config['encoding']
        )
        handler.setFormatter(formatter)
        handler.setLevel(logging.DEBUG)
        self.archive.addHandler(handler)

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(ArchiveHandler, self).get_default_config_help()

        config.update({
            'log_file': 'Path to the logfile',
            'days': 'How many days to store',
            'encoding': '',
            'propagate': 'Pass handled metrics to configured root logger',
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(ArchiveHandler, self).get_default_config()

        config.update({
            'log_file': '',
            'days': 7,
            'encoding': None,
            'propagate': False,
        })

        return config

    def process(self, metric):
        """
        Send a Metric to the Archive.
        """
        # Archive Metric
        self.archive.info(str(metric).strip())



# coding=utf-8

"""
SNMPCollector for Netscaler Metrics

NetScaler is a network appliance manufactured by Citrix providing level 4 load
balancing, firewall, proxy and VPN functions.

"""

import sys
import os
import time
import struct
import re

# Fix Path for locating the SNMPCollector
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__),
                                             '../',
                                             'snmp',
                                             )))

from diamond.metric import Metric
from snmp import SNMPCollector as parent_SNMPCollector


class NetscalerSNMPCollector(parent_SNMPCollector):
    """
    SNMPCollector for Netscaler Metrics
    """

    """
    EntityProtocolType ::=
    INTEGER{    http(0),
                ftp(1),
                tcp(2),
                udp(3),
                sslBridge(4),
                monitor(5),
                monitorUdp(6),
                nntp(7),
                httpserver(8),
                httpclient(9),
                rpcserver(10),
                rpcclient(11),
                nat(12),
                any(13),
                ssl(14),
                dns(15),
                adns(16),
                snmp(17),
                ha(18),
                monitorPing(19),
                sslOtherTcp(20),
                aaa(21),
                secureMonitor(23),
                sslvpnUdp(24),
                rip(25),
                dnsClient(26),
                rpcServer(27),
                rpcClient(28),
                dhcrpa(36),
                sipudp(39),
                dnstcp(44),
                adnstcp(45),
                rtsp(46),
                push(48),
                sslPush(49),
                dhcpClient(50),
                radius(51),
                serviceUnknown(62) }

    EntityState ::=
    INTEGER{    down(1),
                unknown(2),
                busy(3),
                outOfService(4),
                transitionToOutOfService(5),
                up(7),
                transitionToOutOfServiceDown(8) }
    """

    NETSCALER_SYSTEM_GUAGES = {
        "cpuUsage": "1.3.6.1.4.1.5951.4.1.1.41.1.0",
        "memUsage": "1.3.6.1.4.1.5951.4.1.1.41.2.0",
        "surgeQueue": "1.3.6.1.4.1.5951.4.1.1.46.15.0",
        "establishedServerConnections": "1.3.6.1.4.1.5951.4.1.1.46.10.0",
        "establishedClientConnections": "1.3.6.1.4.1.5951.4.1.1.46.12.0"
    }

    NETSCALER_SYSTEM_COUNTERS = {
        "httpTotRequests": "1.3.6.1.4.1.5951.4.1.1.48.67.0"
    }

    NETSCALER_VSERVER_NAMES = "1.3.6.1.4.1.5951.4.1.3.1.1.1"

    NETSCALER_VSERVER_TYPE = "1.3.6.1.4.1.5951.4.1.3.1.1.4"

    NETSCALER_VSERVER_STATE = "1.3.6.1.4.1.5951.4.1.3.1.1.5"

    NETSCALER_VSERVER_GUAGES = {
        "vsvrRequestRate": "1.3.6.1.4.1.5951.4.1.3.1.1.43",
        "vsvrRxBytesRate": "1.3.6.1.4.1.5951.4.1.3.1.1.44",
        "vsvrTxBytesRate": "1.3.6.1.4.1.5951.4.1.3.1.1.45",
        "vsvrCurServicesUp": "1.3.6.1.4.1.5951.4.1.3.1.1.41",
        "vsvrCurServicesDown": "1.3.6.1.4.1.5951.4.1.3.1.1.37",
        "vsvrCurServicesUnknown": "1.3.6.1.4.1.5951.4.1.3.1.1.38",
        "vsvrCurServicesTransToOutOfSvc": "1.3.6.1.4.1.5951.4.1.3.1.1.40"
    }

    NETSCALER_SERVICE_NAMES = "1.3.6.1.4.1.5951.4.1.2.1.1.1"

    NETSCALER_SERVICE_TYPE = "1.3.6.1.4.1.5951.4.1.2.1.1.4"

    NETSCALER_SERVICE_STATE = "1.3.6.1.4.1.5951.4.1.2.1.1.5"

    NETSCALER_SERVICE_GUAGES = {
        "svcRequestRate": "1.3.6.1.4.1.5951.4.1.2.1.1.42",
        "svcSurgeCount": "1.3.6.1.4.1.5951.4.1.2.1.1.10",
        "svcEstablishedConn": "1.3.6.1.4.1.5951.4.1.2.1.1.8",
        "svcActiveConn": "1.3.6.1.4.1.5951.4.1.2.1.1.9",
        "svcCurClntConnections": "1.3.6.1.4.1.5951.4.1.2.1.1.41"
    }

    MAX_VALUE = 18446744073709551615

    def get_default_config_help(self):
        config_help = super(NetscalerSNMPCollector,
                            self).get_default_config_help()
        config_help.update({
            'host': 'netscaler dns address',
            'port': 'Netscaler port to collect snmp data',
            'community': 'SNMP community',
            'exclude_service_type': "list of service types to exclude" +
                                    " (see MIB EntityProtocolType)",
            'exclude_vserver_type': "list of vserver types to exclude" +
                                    " (see MIB EntityProtocolType)"
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(NetscalerSNMPCollector, self).get_default_config()
        config.update({
            'path':     'netscaler',
            'timeout':  15,
            'exclude_service_type': [],
            'exclude_vserver_type': [],
            'exclude_service_state': [],
            'exclude_vserver_state': []
        })
        return config

    def get_string_index_oid(self, s):
        """Turns a string into an oid format is length of name followed by
        name chars in ascii"""
        return (len(self.get_bytes(s)), ) + self.get_bytes(s)

    def get_bytes(self, s):
        """Turns a string into a list of byte values"""
        return struct.unpack('%sB' % len(s), s)

    def collect_snmp(self, device, host, port, community):
        """
        Collect Netscaler SNMP stats from device
        """
        # Log
        self.log.info("Collecting Netscaler statistics from: %s", device)

        # Set timestamp
        timestamp = time.time()

        # Collect Netscaler System OIDs
        for k, v in self.NETSCALER_SYSTEM_GUAGES.items():
            # Get Metric Name and Value
            metricName = '.'.join([k])
            metricValue = int(self.get(v, host, port, community)[v])
            # Get Metric Path
            metricPath = '.'.join(['devices', device, 'system', metricName])
            # Create Metric
            metric = Metric(metricPath, metricValue, timestamp, 0)
            # Publish Metric
            self.publish_metric(metric)

        # Collect Netscaler System Counter OIDs
        for k, v in self.NETSCALER_SYSTEM_COUNTERS.items():
            # Get Metric Name and Value
            metricName = '.'.join([k])
            # Get Metric Path
            metricPath = '.'.join(['devices', device, 'system', metricName])
            # Get Metric Value
            metricValue = self.derivative(metricPath, long(
                self.get(v, host, port, community)[v]), self.MAX_VALUE)
            # Create Metric
            metric = Metric(metricPath, metricValue, timestamp, 0)
            # Publish Metric
            self.publish_metric(metric)

        # Collect Netscaler Services
        serviceNames = [v.strip("\'") for v in self.walk(
            self.NETSCALER_SERVICE_NAMES, host, port, community).values()]

        for serviceName in serviceNames:
            # Get Service Name in OID form
            serviceNameOid = self.get_string_index_oid(serviceName)

            # Get Service Type
            serviceTypeOid = ".".join([self.NETSCALER_SERVICE_TYPE,
                                       self._convert_from_oid(serviceNameOid)])
            serviceType = int(self.get(serviceTypeOid,
                                       host,
                                       port,
                                       community)[serviceTypeOid].strip("\'"))

            # Filter excluded service types
            if serviceType in map(lambda v: int(v),
                                  self.config.get('exclude_service_type')):
                continue

            # Get Service State
            serviceStateOid = ".".join([self.NETSCALER_SERVICE_STATE,
                                        self._convert_from_oid(serviceNameOid)])
            serviceState = int(self.get(serviceStateOid,
                                        host,
                                        port,
                                        community)[serviceStateOid].strip("\'"))

            # Filter excluded service states
            if serviceState in map(lambda v: int(v),
                                   self.config.get('exclude_service_state')):
                continue

            for k, v in self.NETSCALER_SERVICE_GUAGES.items():
                serviceGuageOid = ".".join(
                    [v, self._convert_from_oid(serviceNameOid)])
                # Get Metric Name
                metricName = '.'.join([re.sub(r'\.|\\', '_', serviceName), k])
                # Get Metric Value
                metricValue = int(self.get(serviceGuageOid,
                                           host,
                                           port,
                                           community
                                           )[serviceGuageOid].strip("\'"))
                # Get Metric Path
                metricPath = '.'.join(['devices',
                                       device,
                                       'service',
                                       metricName])
                # Create Metric
                metric = Metric(metricPath, metricValue, timestamp, 0)
                # Publish Metric
                self.publish_metric(metric)

        # Collect Netscaler Vservers
        vserverNames = [v.strip("\'") for v in self.walk(
            self.NETSCALER_VSERVER_NAMES, host, port, community).values()]

        for vserverName in vserverNames:
            # Get Vserver Name in OID form
            vserverNameOid = self.get_string_index_oid(vserverName)

            # Get Vserver Type
            vserverTypeOid = ".".join([self.NETSCALER_VSERVER_TYPE,
                                       self._convert_from_oid(vserverNameOid)])
            vserverType = int(self.get(vserverTypeOid,
                                       host,
                                       port,
                                       community)[vserverTypeOid].strip("\'"))

            # filter excluded vserver types
            if vserverType in map(lambda v: int(v),
                                  self.config.get('exclude_vserver_type')):
                continue

            # Get Service State
            vserverStateOid = ".".join([self.NETSCALER_VSERVER_STATE,
                                        self._convert_from_oid(vserverNameOid)])
            vserverState = int(self.get(vserverStateOid,
                                        host,
                                        port,
                                        community)[vserverStateOid].strip("\'"))

            # Filter excluded vserver state
            if vserverState in map(lambda v: int(v),
                                   self.config.get('exclude_vserver_state')):
                continue

            for k, v in self.NETSCALER_VSERVER_GUAGES.items():
                vserverGuageOid = ".".join(
                    [v, self._convert_from_oid(vserverNameOid)])
                # Get Metric Name
                metricName = '.'.join([re.sub(r'\.|\\', '_', vserverName), k])
                # Get Metric Value
                metricValue = int(self.get(vserverGuageOid,
                                           host,
                                           port,
                                           community
                                           )[vserverGuageOid].strip("\'"))
                # Get Metric Path
                metricPath = '.'.join(['devices',
                                       device,
                                       'vserver',
                                       metricName])
                # Create Metric
                metric = Metric(metricPath, metricValue, timestamp, 0)
                # Publish Metric
                self.publish_metric(metric)



# coding=utf-8

"""
Output the collected values to the debug log channel.
"""

from Handler import Handler


class NullHandler(Handler):
    """
    Implements the abstract Handler class, doing nothing except log
    """

    def process(self, metric):
        """
        Process a metric by doing nothing
        """
        self.log.debug("Metric: %s", str(metric).rstrip().replace(' ', '\t'))

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(NullHandler, self).get_default_config_help()

        config.update({
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(NullHandler, self).get_default_config()

        config.update({
        })

        return config



# coding=utf-8

"""
V. 1.0

JbossApiCollector is a collector that uses JBOSS 7 native API to collect data
Tested on jboss 7.X.X??

Much of the code was borrowed from:

http://bit.ly/XRrCWx
https://github.com/lukaf/munin-plugins/blob/master/jboss7_

References:
https://docs.jboss.org/author/display/AS7/Management+API+reference
http://middlewaremagic.com/jboss/?p=2476

TODO:
This code was made to work with the local system 'curl' command, due to
difficulties getting urllib2 or pycurl to work under the python 2.4 options
successfully doing SSL Digest Authentication.

Plan is to make this code work with newer versions of python and possibly
Requests. (http://docs.python-requests.org/en/latest/)

If possible, please make future updates backwards compatible to call the local
curl as an option.


#### Dependencies

 * java
 * jboss
 * curl
 * json

##### Configuration

# Uses local system curl until can be made to work with either urllib2, pycurl,
or requests (http://docs.python-requests.org/en/latest/)


enabled = True
path_suffix = ""
measure_collector_time = False
interface_regex = ^(.+?)\.
curl_bin = /usr/bin/curl
connect_timeout = 4
hosts = wasadmin:pass@host:9443:https, wasadmin:pass@host:9443:https
curl_options = "-s --digest -L "
ssl_options = "--sslv3 -k"
connector_stats = True | False
connector_options =  http, ajp
app_stats = True | False
jvm_memory_stats = True | False
jvm_buffer_pool_stats = True | False
jvm_memory_pool_stats = True | False
jvm_gc_stats = True | False
jvm_thread_stats = True | False


"""

import diamond.collector
import os
import re
import subprocess

try:
    import json
except ImportError:
    import simplejson as json


# Setup a set of VARs
# Set this for use in curl request
header = '"Content-Type: application/json"'

operational_type = [
    'app',
    'web',
    'jvm',
    'thread_pool',
]

web_stats = [
    'errorCount',
    'requestCount',
    'bytesReceived',
    'bytesSent',
    'processingTime',
]

memory_types = [
    'init',
    'used',
    'committed',
    'max',
]

buffer_pool_types = [
    'count',
    'memory-used',
]

thread_types = [
    'thread-count',
    'daemon-thread-count'
]

memory_topics = [
    'heap-memory-usage',
    'non-heap-memory-usage',
]

gc_types = [
    'collection-count',
    'collection-time',
]


class JbossApiCollector(diamond.collector.Collector):

    def process_config(self):
        super(JbossApiCollector, self).process_config()
        if self.config['hosts'].__class__.__name__ != 'list':
            self.config['hosts'] = [self.config['hosts']]

        # get the params for each host
        if 'host' in self.config:
            hoststr = "%s:%s@%s:%s:%s" % (
                self.config['user'],
                self.config['password'],
                self.config['host'],
                self.config['port'],
                self.config['proto'],
            )
            self.config['hosts'].append(hoststr)

        if type(self.config['connector_options']) is not list:
            self.config['connector_options'] = [
                self.config['connector_options']]

    def get_default_config_help(self):
        # Need to update this when done to help explain details when running
        # diamond-setup.
        config_help = super(JbossApiCollector, self).get_default_config_help()
        config_help.update({
            'curl_bin': 'Path to system curl executable',
            'hosts': 'List of hosts to collect from. Format is yourusername:yourpassword@host:port:proto',  # NOQA
            'app_stats': 'Collect application pool stats',
            'jvm_memory_pool_stats': 'Collect JVM memory-pool stats',
            'jvm_buffer_pool_stats': 'Collect JVM buffer-pool stats',
            'jvm_memory_stats': 'Collect JVM basic memory stats',
            'jvm_gc_stats': 'Collect JVM garbage-collector stats',
            'jvm_thread_stats': 'Collect JVM thread stats',
            'thread_pool_stats': 'Collect JBoss thread pool stats',
            'connector_stats': 'Collect HTTP and AJP Connector stats',
            'connector_options': 'Types of connectors to collect'
        })
        return config_help

    def get_default_config(self):

        # Initialize default config
        config = super(JbossApiCollector, self).get_default_config()
        config.update({
            'path': 'jboss',
            'curl_bin': '/usr/bin/curl',
            'connect_timeout': '4',
            'ssl_options': '--sslv3 -k',
            'curl_options': '-s --digest -L ',
            'interface_regex': '^(.+?)\.',  # matches up to first "."
            'hosts': [],
            'app_stats': 'True',
            'connector_options': ['http', 'ajp'],
            'jvm_memory_pool_stats': 'True',
            'jvm_buffer_pool_stats': 'True',
            'jvm_memory_stats': 'True',
            'jvm_gc_stats': 'True',
            'jvm_thread_stats': 'True',
            'connector_stats': 'True',
            'thread_pool_stats': 'True'
        })
        # Return default config
        return config

    def get_stats(self, current_host, current_port, current_proto, current_user,
                  current_pword):

        if not os.access(self.config['curl_bin'], os.X_OK):
            self.log.error("%s is not executable or does not exist.",
                           self.config['curl_bin'])

        # Check if there is a RegEx to perform on the interface names
        if self.config['interface_regex'] != '':
            interface = self.string_regex(self.config['interface_regex'],
                                          current_host)

        else:
            # Clean up any possible extra "."'s in the interface, keeps
            # graphite from creating directories
            interface = self.string_fix(current_host)

        for op_type in operational_type:
            output = self.get_data(op_type, current_host, current_port,
                                   current_proto, current_user, current_pword)
            if op_type == 'app' and self.config['app_stats'] == 'True':
                if output:
                    # Grab the pool stats for each Instance
                    for instance in output['result']['data-source']:
                        datasource = output['result']['data-source'][instance]
                        for metric in datasource['statistics']['pool']:
                            metricName = '%s.%s.%s.statistics.pool.%s' % (
                                interface, op_type, instance, metric)
                            metricValue = datasource[
                                'statistics']['pool'][metric]
                            self.publish(metricName, float(metricValue))

            if (op_type == 'thread_pool' and
                    self.config['thread_pool_stats'] == 'True' and output):
                # Grab the stats from each thread pool type
                for pool_type in output['result']:
                    if output['result'][pool_type]:
                        pool_types = output['result'][pool_type]
                        for pool in pool_types:
                            for metric in pool_types[pool]:
                                metricName = '%s.%s.%s.%s.statistics.%s' % (
                                    interface, op_type, pool_type,
                                    pool, metric)
                                metricValue = pool_types[pool][metric]
                                if self.is_number(metricValue):
                                    self.publish(metricName, float(metricValue))

            if op_type == 'web' and self.config['connector_stats'] == 'True':
                if output:
                    # Grab http and ajp info (make these options)
                    for c_type in self.config['connector_options']:
                        for metric in web_stats:
                            metricName = ('%s.%s.connector.%s.%s' %
                                          (interface,
                                           op_type,
                                           c_type,
                                           metric))
                            connector = output['result']['connector']
                            metricValue = connector[c_type][metric]
                            self.publish(metricName, float(metricValue))

            if op_type == 'jvm':
                if output:
                    if self.config['jvm_memory_pool_stats'] == 'True':
                        # Grab JVM memory pool stats
                        mempool = output['result']['type']['memory-pool']
                        for pool_name in mempool['name']:
                            for metric in memory_types:
                                metricName = ('%s.%s.%s.%s.%s.%s' %
                                              (interface,
                                               op_type,
                                               'memory-pool',
                                               pool_name,
                                               'usage',
                                               metric))
                                metricValue = mempool['name'][pool_name][
                                    'usage'][metric]
                                self.publish(metricName, float(metricValue))

                    # Grab JVM buffer-pool stats
                    if self.config['jvm_buffer_pool_stats'] == 'True':
                        bufferpool = output['result']['type']['buffer-pool']
                        for pool in bufferpool['name']:
                            for metric in buffer_pool_types:
                                metricName = ('%s.%s.%s.%s.%s' %
                                              (interface,
                                               op_type,
                                               'buffer-pool',
                                               pool,
                                               metric))
                                metricValue = bufferpool['name'][pool][metric]
                                self.publish(metricName, float(metricValue))

                    # Grab basic memory stats
                    if self.config['jvm_memory_stats'] == 'True':
                        for mem_type in memory_topics:
                            for metric in memory_types:
                                metricName = ('%s.%s.%s.%s.%s' %
                                              (interface,
                                               op_type,
                                               'memory',
                                               mem_type,
                                               metric))
                                memory = output['result']['type']['memory']
                                metricValue = memory[mem_type][metric]
                                self.publish(metricName, float(metricValue))

                    # Grab Garbage collection stats
                    if self.config['jvm_gc_stats'] == 'True':
                        garbage = output['result']['type']['garbage-collector']
                        for gc_name in garbage['name']:
                            for metric in gc_types:
                                metricName = ('%s.%s.%s.%s.%s' %
                                              (interface,
                                               op_type,
                                               'garbage-collector',
                                               gc_name,
                                               metric))
                                metricValue = garbage['name'][gc_name][metric]
                                self.publish(metricName, float(metricValue))

                    # Grab threading stats
                    if self.config['jvm_thread_stats'] == 'True':
                        for metric in thread_types:
                            metricName = ('%s.%s.%s.%s' %
                                          (interface,
                                           op_type,
                                           'threading',
                                           metric))
                            threading = output['result']['type']['threading']
                            metricValue = threading[metric]
                            self.publish(metricName, float(metricValue))

        return True

    def get_data(self, op_type, current_host, current_port, current_proto,
                 current_user, current_pword):
        output = {}
        if op_type == 'app':
            data = ('{"operation":"read-resource", ' +
                    '"include-runtime":"true", ' +
                    '"recursive":"true", ' +
                    '"address":["subsystem","datasources"]}')

        if op_type == 'thread_pool':
            data = ('{"operation":"read-resource", "include-runtime":"true", ' +
                    '"recursive":"true" , "address":["subsystem","threads"]}')

        if op_type == 'web':
            data = ('{"operation":"read-resource", ' +
                    '"include-runtime":"true", ' +
                    '"recursive":"true", ' +
                    '"address":["subsystem","web"]}')

        if op_type == 'jvm':
            data = ('{"operation":"read-resource", ' +
                    '"include-runtime":"true", ' +
                    '"recursive":"true", ' +
                    '"address":["core-service","platform-mbean"]}')

        the_cmd = (("%s --connect-timeout %s %s %s %s://%s:%s/management " +
                    "--header %s -d '%s' -u %s:%s") % (
            self.config['curl_bin'], self.config['connect_timeout'],
            self.config['ssl_options'], self.config['curl_options'],
            current_proto, current_host, current_port, header, data,
            current_user, current_pword))

        try:
            attributes = subprocess.Popen(the_cmd, shell=True,
                                          stdout=subprocess.PIPE
                                          ).communicate()[0]
            output = json.loads(attributes)
        except Exception, e:
            self.log.error("JbossApiCollector: There was an exception %s", e)
            output = ''
        return output

    def is_number(self, value):
        return (isinstance(value, (int, long, float)) and
                not isinstance(value, bool))

    def string_fix(self, s):
        return re.sub(r"[^a-zA-Z0-9_]", "_", s)

    def string_regex(self, pattern, s):
        tmp_result = re.match(pattern, s)
        return tmp_result.group(1)

    def collect(self):

        for host in self.config['hosts']:
            matches = re.search(
                '^([^:]*):([^@]*)@([^:]*):([^:]*):?(.*)', host)

            if not matches:
                continue

            current_host = matches.group(3)
            current_port = int(matches.group(4))
            current_proto = matches.group(5)
            current_user = matches.group(1)
            current_pword = matches.group(2)

            # Call get_stats for each instance of jboss
            self.get_stats(current_host, current_port, current_proto,
                           current_user, current_pword)

        return True



# coding=utf-8

"""
Collect out of band stats from ntp

Uses output from ntpdate:

```
$ ntpdate -q pool.ntp.org
server 12.34.56.1, stratum 2, offset -0.000277, delay 0.02878
server 12.34.56.2, stratum 1, offset -0.000128, delay 0.02896
server 12.34.56.3, stratum 2, offset 0.000613, delay 0.02870
server 12.34.56.4, stratum 2, offset -0.000351, delay 0.02864
31 Apr 12:00:00 ntpdate[12]: adjust time server 12.34.56.2 offset -0.000128 sec
$
```

#### Dependencies

    * /usr/sbin/ntpdate
    * subprocess

"""

import subprocess

import diamond.collector
from diamond.collector import str_to_bool
from diamond import convertor


class NtpCollector(diamond.collector.ProcessCollector):

    def get_default_config_help(self):
        config_help = super(NtpCollector, self).get_default_config_help()
        config_help.update({
            'bin':      'Path to ntpdate binary',
            'ntp_pool': 'NTP Pool address',
            'precision': 'Number of decimal places to report to',
            'time_scale': 'Time unit to report offset in',
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(NtpCollector, self).get_default_config()
        config.update({
            'bin':      self.find_binary('/usr/sbin/ntpdate'),
            'ntp_pool': 'pool.ntp.org',
            'path':     'ntp',
            'precision': 0,
            'time_scale': 'milliseconds',
        })
        return config

    def get_ntpdate_stats(self):
        output = self.run_command(['-q', self.config['ntp_pool']])

        data = {}
        data['server.count'] = {'val': 0, 'precision': 0}

        for line in output[0].splitlines():
            # Only care about best choice not all servers
            if line.startswith('server'):
                data['server.count']['val'] += 1
                continue

            parts = line.split()

            # Make sure we have the correct output
            # Sample of line: 31 Apr 12:00:00 ntpdate[12345]: adjust time \
            #   server 123.456.789.2 offset -0.000123 sec
            if len(parts) != 11:
                self.log.error('NtpCollector: Output of ntpdate was %s words '
                               'long but was expected to be 11' % len(parts))
                self.log.debug('NtpCollector: ntpdate output was %s' % parts)
                continue

            # offset is in seconds, convert is to nanoseconds and miliseconds
            offset_in_s = float(parts[9])

            # Convert to the requested time unit
            offset = convertor.time.convert(offset_in_s,
                                            's',
                                            self.config['time_scale'])

            # Determine metric namespace based on given time unit
            metric_name = 'offset.%s' % self.config['time_scale']

            data[metric_name] = {'val': offset,
                                 'precision': self.config['precision']}

        return data.items()

    def collect(self):
        for stat, v in self.get_ntpdate_stats():
            self.publish(stat, v['val'], precision=v['precision'])



# coding=utf-8

"""
This collector uses the [ipmitool](http://openipmi.sourceforge.net/) to read
hardware sensors from servers
using the Intelligent Platform Management Interface (IPMI). IPMI is very common
with server hardware but usually not available in consumer hardware.

#### Dependencies

 * [ipmitool](http://openipmi.sourceforge.net/)

"""

import diamond.collector
from diamond.collector import str_to_bool
from subprocess import Popen, PIPE
import os
import getpass


class IPMISensorCollector(diamond.collector.Collector):

    def get_default_config_help(self):
        config_help = super(IPMISensorCollector,
                            self).get_default_config_help()
        config_help.update({
            'bin': 'Path to the ipmitool binary',
            'use_sudo': 'Use sudo?',
            'sudo_cmd': 'Path to sudo',
            'thresholds': 'Collect thresholds as well as reading',
            'delimiter': 'Parse blanks in sensor names into a delimiter'
        })
        return config_help

    def get_default_config(self):
        """
        Returns the default collector settings
        """
        config = super(IPMISensorCollector, self).get_default_config()
        config.update({
            'bin':              '/usr/bin/ipmitool',
            'use_sudo':         False,
            'sudo_cmd':         '/usr/bin/sudo',
            'path':             'ipmi.sensors',
            'thresholds':       False,
            'delimiter':        '.'
        })
        return config

    def parse_value(self, value):
        """
        Convert value string to float for reporting
        """
        value = value.strip()

        # Skip missing sensors
        if value == 'na':
            return None

        # Try just getting the float value
        try:
            return float(value)
        except:
            pass

        # Next best guess is a hex value
        try:
            return float.fromhex(value)
        except:
            pass

        # No luck, bail
        return None

    def collect(self):
        use_sudo = str_to_bool(self.config['use_sudo'])
        if ((not os.access(self.config['bin'], os.X_OK) or
             (use_sudo and
              not os.access(self.config['sudo_cmd'], os.X_OK)))):
            return False

        command = [self.config['bin'], 'sensor']

        if use_sudo and getpass.getuser() != 'root':
            command.insert(0, self.config['sudo_cmd'])

        p = Popen(command, stdout=PIPE).communicate()[0][:-1]

        for i, v in enumerate(p.split("\n")):
            data = v.split("|")
            try:
                # Complex keys are fun!
                metric_name = data[0].strip()
                metric_name = metric_name.replace(".", "_")
                metric_name = metric_name.replace(" ",
                                                  self.config['delimiter'])
                metrics = []

                # Each sensor line is a column seperated by a | with the
                # following descriptions:
                # 1. Sensor ID
                # 2. Sensor Reading
                # 3. Units
                # 4. Status
                # 5. Lower Non-Recoverable
                # 6. Lower Critical
                # 7. Lower Non-Critical
                # 8. Upper Non-Critical
                # 9. Upper Critical
                # 10. Upper Non-Recoverable

                if not self.config['thresholds']:
                    metrics.append((metric_name, self.parse_value(data[1])))
                else:
                    metrics.append((metric_name + ".Reading",
                                    self.parse_value(data[1])))
                    metrics.append((metric_name + ".Lower.NonRecoverable",
                                    self.parse_value(data[4])))
                    metrics.append((metric_name + ".Lower.Critical",
                                    self.parse_value(data[5])))
                    metrics.append((metric_name + ".Lower.NonCritical",
                                    self.parse_value(data[6])))
                    metrics.append((metric_name + ".Upper.NonCritical",
                                    self.parse_value(data[7])))
                    metrics.append((metric_name + ".Upper.Critical",
                                    self.parse_value(data[8])))
                    metrics.append((metric_name + ".Upper.NonRecoverable",
                                    self.parse_value(data[9])))

                [self.publish(name, value)
                 for (name, value) in metrics
                 if value is not None]

            except ValueError:
                continue
            except IndexError:
                continue

        return True
